@@ -33,9 +33,11 @@ static GtkWidget *new_folder_entry;
 static GCallback data_from_jukebox_to_hd (GtkButton *button,
 					  gpointer data)
 {
-  static jb2hd_data_thread_arg_t jb2hd_data_thread_args;
+  jb2hd_data_thread_arg_t *jb2hd_data_thread_args;
   GList *metalist = get_all_metadata_from_selection(JBDATA_LIST);
 
+  jb2hd_data_thread_args = (jb2hd_data_thread_arg_t *) malloc(sizeof (jb2hd_data_thread_arg_t));
+
   if (jukebox_locked) {
     create_error_dialog(_("Jukebox busy"));
     destroy_metalist(metalist);
@@ -89,12 +91,12 @@ static GCallback data_from_jukebox_to_hd (GtkButton *button,
 #endif
     gtk_widget_show_all(dialog);
     
-    jb2hd_data_thread_args.dialog = dialog;
-    jb2hd_data_thread_args.label = label2;
-    jb2hd_data_thread_args.metalist = metalist;
+    jb2hd_data_thread_args->dialog = dialog;
+    jb2hd_data_thread_args->label = label2;
+    jb2hd_data_thread_args->metalist = metalist;
     jukebox_locked = TRUE;
     cancel_jukebox_operation = FALSE;
-    g_thread_create(jb2hd_data_thread,(gpointer) &jb2hd_data_thread_args, FALSE, NULL);
+    g_thread_create(jb2hd_data_thread,(gpointer) jb2hd_data_thread_args, FALSE, NULL);
   }
 }
 
@@ -102,9 +104,11 @@ static GCallback data_from_jukebox_to_hd (GtkButton *button,
 /* Move the files off the Harddisk and into the Jukebox */
 static GCallback data_from_hd_to_jukebox()
 {
-  static hd2jb_data_thread_arg_t hd2jb_data_thread_args;
+  hd2jb_data_thread_arg_t *hd2jb_data_thread_args;
   GList *metalist = get_all_metadata_from_selection(HDDATA_LIST);
 
+  hd2jb_data_thread_args = (hd2jb_data_thread_arg_t *) malloc(sizeof (hd2jb_data_thread_arg_t));
+
   if (jukebox_locked) {
     create_error_dialog(_("Jukebox busy"));
     destroy_metalist(metalist);
@@ -158,12 +162,12 @@ static GCallback data_from_hd_to_jukebox()
 #endif
     gtk_widget_show_all(dialog);
     
-    hd2jb_data_thread_args.dialog = dialog;
-    hd2jb_data_thread_args.label = label2;
-    hd2jb_data_thread_args.metalist = metalist;
+    hd2jb_data_thread_args->dialog = dialog;
+    hd2jb_data_thread_args->label = label2;
+    hd2jb_data_thread_args->metalist = metalist;
     jukebox_locked = TRUE;
     cancel_jukebox_operation = FALSE;
-    g_thread_create(hd2jb_data_thread,(gpointer) &hd2jb_data_thread_args, FALSE, NULL);
+    g_thread_create(hd2jb_data_thread,(gpointer) hd2jb_data_thread_args, FALSE, NULL);
   }
 }
 
