@@ -101,6 +101,12 @@ const gint chomp_string_count =
 const gint replacement_string_count = 
 ((gint)(sizeof(replacement_strings)/sizeof(gchar*))-1);
 
+/**
+ * This routine composes a suitable filename for a track file
+ * based on user configuration.
+ * @param meta the metadata to use for constructing the filename
+ * @return a UTF-8 string with the desired filename
+ */
 gchar *compose_filename(metadata_t *meta)
 {
   gchar *returnname;
@@ -121,13 +127,10 @@ gchar *compose_filename(metadata_t *meta)
   returnname = compose_metadata_string(meta, template, TRUE);
   g_free(template);
   returnname = stringcat(returnname, ".");
-  tmpcodec = g_strdup(meta->codec);
-  g_strdown(tmpcodec);
+  tmpcodec = g_utf8_strdown(meta->codec,-1);
   returnname = stringcat(returnname, tmpcodec);
   g_free(tmpcodec);
   /* g_print("Returnname: %s\n", returnname); */
-  /* Its not good to have these in a filename */
-  replacechar(returnname, G_SEARCHPATH_SEPARATOR, '-');
   return returnname;
 }
 
@@ -270,11 +273,9 @@ static gchar *remove_crap_from_filename(gchar *instring)
   /* Return on empty string */
   if (!instring)
     return instring;
-  /* Make a working copy */
-  string = g_strdup(instring);
+  /* Make a lowercase working copy */
+  string = g_utf8_strdown(instring,-1);
   g_free(instring);
-  /* Lowercase */
-  g_strdown(string);
   /* Remove any underscores */
   replacechar(string, '_', ' ');
   /* Remove URL encoding present in some MP3 file
@@ -476,7 +477,6 @@ static gchar *get_title (gchar *file)
       tmp3 = *tmp2;
       tmp2++;
     }
-    g_print("%s\n", tmp3);
   }
   g_strfreev(tmp);
   return g_strdup(file);
