@@ -244,6 +244,10 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
 	int i;
 
 	if (!anyfile) {
+	  /* Must be more atleast one byte */
+	  if (size == 0)
+	    continue;
+	  /* Must have sane filename */
 	  if (strlen(dir_entry->d_name) < 4)
 	    continue;
 	  strcpy(tmp_codec, dir_entry->d_name+strlen(dir_entry->d_name)-3);
@@ -251,7 +255,14 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
 	  /* Only display valid files */
 	  if (g_ascii_strcasecmp(tmp,"mp3") &&
 	      g_ascii_strcasecmp(tmp,"wav") &&
-	      g_ascii_strcasecmp(tmp,"wma")) {
+	      g_ascii_strcasecmp(tmp,"wma")
+#ifdef HAVE_LIBMTP
+	      // Additional music codecs supported by libmtp
+	      &&
+	      g_ascii_strcasecmp(tmp,"ogg") &&
+	      g_ascii_strcasecmp(tmp,"mp4")
+#endif
+	      ) {
 	    g_free(tmp);
 	    continue;
 	  }
@@ -314,6 +325,7 @@ static GList *get_metadata_for_dirlist(GList *inlist,
     gchar *tmp;
     guint size;
     gint i;
+    gchar fileext[5];
     
     meta = new_metadata_t();
     /* Full path -> ID column */
@@ -344,19 +356,29 @@ static GList *get_metadata_for_dirlist(GList *inlist,
       gtk_label_set_text(GTK_LABEL(args->label), tmp);
       gdk_threads_leave();
     }
+    /* Find codec/file extension first */
+    if (strlen(meta->path) > 5) {
+      gchar *tmp = meta->path;
+
+      // Accept 3 or 4 character extensions
+      if (tmp[strlen(tmp)-4] == '.') {
+	strcpy(fileext, tmp+strlen(tmp)-3);
+      } else if (tmp[strlen(tmp)-5] == '.') {
+	strcpy(fileext, tmp+strlen(tmp)-4);
+      }
+      // Uppercase it.
+      g_strup(fileext);
+      meta->codec = g_strdup(fileext);
+    }
     if (listtype == HDDATA_LIST) {
-      /* For data files, provide basic data only 
-       * nothing additional is needed.
+      /* 
+       * For data files, provide basic data nothing additional is needed.
+       * The file extension will be used to map the apropriate transfer
+       * type later.
        */
     } else {
       /* For music files, fill in row by using libid3tag and friends */
       if (meta->size != 0) {
-	/* Find codec first */
-	static gchar codec[4];
-	
-	strcpy(codec, meta->path+strlen(meta->path)-3);
-	g_strup(codec);
-	meta->codec = g_strdup(codec);
 	if (!strcmp(meta->codec,"MP3")) {
 	  /* Gets the ID3 tag if any */
 	  if (gnomad_debug != 0) {
@@ -381,6 +403,13 @@ static GList *get_metadata_for_dirlist(GList *inlist,
 	if (!strcmp(meta->codec,"WMA")) {
 	  get_tag_for_wmafile(meta);
 	}
+#ifdef HAVE_LIBMTP
+	// Implement getting track metadata for ogg and mp4 here.
+	if (!strcmp(meta->codec,"OGG")) {
+	}
+	if (!strcmp(meta->codec,"MP4")) {
+	}
+#endif
 	/* 
 	 * Gather additional information from the path and file name
 	 * if this preference is set. 
