@@ -20,11 +20,25 @@ Boston, MA 02111-1307, USA.
 #ifndef FILESYSTEMH_INCLUDED
 #define FILESYSTEMH_INCLUDED 1
 
+#include <gtk/gtk.h>
 #include "metadata.h"
 
-/* Mode flag for created files and dirs */
+/* Mode flags for files and dirs */
+#ifdef G_OS_WIN32
+#define READONLY_FOPEN_STRING ("rb")
+#define READPLUS_FOPEN_STRING ("rb+")
+#define READONLY_FLAGS (O_RDONLY | O_BINARY)
+#define WRITEONLY_FLAGS (O_WRONLY | O_BINARY)
+#define CREATE_FILEMODE (S_IRUSR | S_IWUSR)
+#define CREATE_DIRMODE (S_IRUSR | S_IWUSR | S_IXUSR)
+#else
+#define READONLY_FOPEN_STRING ("r")
+#define READPLUS_FOPEN_STRING ("rb")
+#define READONLY_FLAGS (O_RDONLY)
+#define WRITEONLY_FLAGS (O_WRONLY)
 #define CREATE_FILEMODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
 #define CREATE_DIRMODE (S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+#endif
 
 /* Local type for the metadata read thread */
 typedef struct 
@@ -38,8 +52,9 @@ typedef struct
 } dirfill_thread_arg_t;
 
 /* Exported functions */
-gchar *filename_toutf8(const gchar *instr);
-gchar *filename_fromutf8(const gchar *instr);
+guint64 get_64bit_file_size(const gchar * const path);
+gchar *filename_toutf8(const gchar * const instr);
+gchar *filename_fromutf8(const gchar * const instr);
 GList *get_metadata_dir(listtype_t listtype, 
 			gchar *path, 
 			dirfill_thread_arg_t *args);
@@ -48,7 +63,7 @@ gchar *get_current_dir();
 gboolean is_directory(gchar *dir);
 void change_directory(gchar *dir);
 void delete_files(GList *filelist);
-gboolean clone_and_strip_id3(gchar *source, gchar *dest);
+gboolean clone_and_strip_id3(const gchar *const source, const gchar * const dest);
 gint get_prefsfile_fd(gboolean write);
 void close_prefsfile(void);
 void rewrite_prefsfile(gchar *prefs);
