@@ -227,7 +227,7 @@ static void about_box(gpointer data)
   gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), VERSION);
   gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about), copystring);
   gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about), aboutstring);
-  gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about), "GPL version 2");
+  gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about), "GPL 2+");
   gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about), "http://gnomad2.sourceforge.net/");
   gtk_about_dialog_set_website_label(GTK_ABOUT_DIALOG(about), "Gnomad Homepage");
   gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(about), authors);
@@ -273,7 +273,7 @@ void scan_jukebox(gpointer data)
 				   GTK_MESSAGE_INFO,
 				   GTK_BUTTONS_CLOSE,
 				   _("Scanning jukebox library"));
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Retrieveing metadata from jukebox"));
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Retrieving metadata from jukebox"));
   g_signal_connect_object(GTK_OBJECT(dialog),
 			  "delete_event",
 			  G_CALLBACK(cancel_jukebox_operation_click),
