@@ -34,8 +34,8 @@ Boston, MA 02111-1307, USA.
 static id3_utf8_t *charset_to_utf8(const id3_latin1_t *str)
 {
   id3_utf8_t *tmp;
-  
-  tmp = (id3_utf8_t *) g_convert((gchar *) str,-1,"UTF-8","ISO-8859-1",NULL,NULL,NULL);
+   
+ tmp = (id3_utf8_t *) g_convert((gchar *) str,-1,"UTF-8","ISO-8859-1",NULL,NULL,NULL);
   return (id3_utf8_t *) tmp;
 }
 
@@ -47,26 +47,41 @@ static id3_latin1_t *charset_from_utf8(const id3_utf8_t *str)
   return (id3_latin1_t *) tmp;
 }
 
+/**
+ * This function checks if a given file has an ID3 tag 
+ * (either V1 or V2, either header and footer) or not.
+ *
+ * @param path a path to the file to check
+ * @return true if the file has an ID3 tag, false otherwise
+ */
 static gboolean hasid3tag(gchar *path)
 {
   gint fd;
   guchar tag[3];
   gboolean result = FALSE;
   
-  fd = (gint) open(path, O_RDONLY, 0);
+#if !GLIB_CHECK_VERSION(2,6,0)  
+  fd = (gint) open(path, READONLY_FLAGS, 0);
+#else
+  fd = (gint) g_open(path, READONLY_FLAGS, 0);
+#endif
   if (fd >= 0) {
     /* First check for a header */
     if(read(fd, tag, 3) == 3) {
-      // g_print("Checked for header ID3v2 tag...\n");
+      if (gnomad_debug != 0) {
+	g_print("Checking for header ID3v2 tag...\n");
+      }
       if (tag[0] == 'I' &&
 	  tag[1] == 'D' &&
 	  tag[2] == '3') {
-	result = TRUE;
+	result = TRUE;	
       } else {
 	/* Seek to end, check for footer */
 	if (lseek(fd, -10, SEEK_END) > 0) {
 	  if (read(fd, tag, 3) == 3) {
-	    // g_print("Checked for footer ID3v2 tag...\n");
+	    if (gnomad_debug != 0) {
+	      g_print("Checking for footer ID3v2 tag...\n");
+	    }
 	    if (tag[0] == '3' &&
 		tag[1] == 'D' &&
 		tag[2] == 'I') {
@@ -75,7 +90,9 @@ static gboolean hasid3tag(gchar *path)
 	      /* Seek to end, and check for ID3v1 tag */
 	      if (lseek(fd, -128, SEEK_END) > 0) {
 		if (read(fd, tag, 3) == 3) {
-		  // g_print("Checked for ID3v1 tag...\n");
+		  if (gnomad_debug != 0) {
+		    g_print("Checking for ID3v1 tag...\n");
+		  }
 		  if (tag[0] == 'T' &&
 		      tag[1] == 'A' &&
 		      tag[2] == 'G') {
@@ -90,136 +107,12 @@ static gboolean hasid3tag(gchar *path)
     }
     close(fd);
   }
-  // g_print(result ? "%s has tag...\n" : "%s has no tag...\n", path);
+  if (gnomad_debug != 0) {
+    g_print(result ? "%s has ID3 tag.\n" : "%s has no ID3 tag.\n", path);
+  }
   return result;
 }
 
-/*
-static gboolean addid3tag(gchar *path)
-{
-  // File descriptors...
-  gint f1, f2, f3;
-  gboolean retval = FALSE;
-  gchar template[128];
-  
-  // g_print("Adding a new ID3 tag to %s\n", path);
-
-  f1 = (gint) open(path, O_RDONLY, 0);
-  if (f1 < 0)
-    return FALSE;
-  // g_print("Opened file f1...\n");
-
-  // template = g_build_filename(g_get_tmp_dir(), "gnomadXXXXXX");
-  strcpy(template, g_get_tmp_dir());
-  template[strlen(template)+1] = '\0';
-  template[strlen(template)] = G_DIR_SEPARATOR;
-  strcat(template, "gnomadXXXXXX");
-
-  // Temporary file
-  f2 = g_mkstemp(template);
-  // g_print("Opened temporary file f2...\n");
-  if (f2 >= 0) {
-    register gchar *buffer;
-    gchar blankv2tag[0x0c0a];
-    guint bufsiz;
-    // A blank ID3v2 tag with title set to "x"
-    gchar v2head[] = {0x49, 0x44, 0x33, 0x03, 0x00, 0x00, 0x00, 0x00, 
-		      0x0c, 0x00, // size 0x0100
-		      0x54, 0x50, 0x45, 0x31, 0x00, 0x00, 0x00, 0x0a,
-		      0x00, 0x00, 0x00, 0x78};
-    gchar blankv1tag[] = {'T', 'A', 'G', 
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-			  0x00, 0x00, 0x00, 0x00, 0x00 };
-
-    // Allocate a copying buffer
-    for (bufsiz = 0x8000; bufsiz >= 128; bufsiz >>= 1) {
-      buffer = (gchar *) g_malloc(bufsiz);
-      if (buffer)
-	break;
-    }
-    // g_print("Allocated a buffer of %d bytes...\n", bufsiz);
-
-    // Add a dummy tag
-    // g_print("Adding a dummy ID3v2 tag...\n");
-    memset(blankv2tag, 0, 0x0c0a);
-    memcpy(blankv2tag, v2head, 22);
-    if (0x0c0a == write(f2,blankv2tag,0x0c0a)) {
-      // Copy the entire file
-      //g_print("Copying the original file...\n");
-      while (1) {
-	register guint n;
-	
-	n = read(f1,buffer,bufsiz);
-	if (n == -1)
-	  break;
-	if (n == 0) {
-	  retval = TRUE;
-	  break;
-	}
-	if (n != write(f2,buffer,n))
-	  break;
-      }
-    }
-    // g_print("Adding a dummy ID3v1 tag...\n");
-    write(f2,blankv1tag,128);
-  
-    if (retval) {
-      close(f1);
-      // Rewind the temporary file
-      lseek(f2, 0, SEEK_SET);
-      // Then copy the file back again
-      // g_print("Copying the file back...\n");
-      f3 = (gint) creat(path, (mode_t) CREATE_FILEMODE);
-      if (f3) {
-	// g_print("Creat() on original file succeeded...\n");
-	while (1) {
-	  register guint n;
-	  
-	  n = read(f2,buffer,bufsiz);
-	  if (n == -1)
-	    break;
-	  if (n == 0) {
-	    retval = TRUE;
-	    break;
-	  }
-	  if (n != write(f3,buffer,n))
-	    break;
-	}
-	close(f3);
-      } else {
-	// g_print("Error copying back!\n");
-	retval = FALSE;
-      }
-      close(f2);
-    } else {
-      close(f1);
-      close(f2);
-    }
-    // g_print("Deleting %s\n", template);
-    unlink(template);
-    // Free the buffer
-    g_free(buffer);
-    return retval;
-  }
-  close(f1);
-  return FALSE;
-}
-*/
-
 /*
  * This function checks if a RIFF header exists at the beginning of 
  * the *data buffer. In that case its size is returned. The size
@@ -229,6 +122,11 @@ static guint riff_header_size(gchar *data, guint bufsize)
 {
   guint retsize = 0;
 
+  // Too small to be a RIFF header
+  if (bufsize < 20) {
+    return 0;
+  }
+
   // Remove any RIFF header too (EVIL GARBAGE!)
   if (data[0] == 'R' &&
       data[1] == 'I' &&
@@ -256,7 +154,9 @@ static guint riff_header_size(gchar *data, guint bufsize)
       if (!strcmp(chunk_type, "data")) {
 	// The 'data' chunk is the vital one. We need not scan further.
 	retsize = p;
-	// g_print("Found a RIFF tag of size %d bytes\n", retsize);
+	if (gnomad_debug != 0) {
+	  g_print("Found a RIFF tag of size %d bytes (%x)\n", retsize, retsize);
+	}
 	break;
       }
       p += chunk_size;
@@ -268,6 +168,8 @@ static guint riff_header_size(gchar *data, guint bufsize)
 
 /*****************************************************************************
  * ID3TAG interface
+ * Many parts of this code copied in from gtkpod or plagiated, I confess.
+ * Acknowledgements to Jorg Schuler for that...
  *****************************************************************************/
 
 /*
@@ -359,8 +261,12 @@ setFrameText(struct id3_tag *tag,
   
   if (data == NULL)
     return;
-  
-  // g_print("updating id3 (enc: %d): %s: %s\n", encoding, frame_name, data);
+
+  if (gnomad_debug != 0) {  
+    g_print("Updating id3 frame (enc: %d): %s: ", encoding, frame_name, data);
+    hexdump((unsigned char *) data, strlen(data));
+    g_print(" %s\n", data);
+  }
   
   /*
    * An empty string removes the frame altogether.
@@ -382,22 +288,32 @@ setFrameText(struct id3_tag *tag,
   } else if (!override) {
     /* Do not overwrite old tags then */
     return;
+  } else {
+    // Needed for unicode?
+    id3_tag_detachframe(tag, frame);
+    frame = id3_frame_new(frame_name);
+    id3_tag_attachframe(tag, frame);
   }
 
+  /* Set the encoding - always field 0 */
+  field = id3_frame_field(frame, 0);
+  id3_field_settextencoding(field, encoding);
+
+  /* Modify the default text field type */
   if (frame_name == ID3_FRAME_COMMENT) {
+    /* Get the latin-1 string list, convert it to a Unicode list */
     field = id3_frame_field (frame, 3);
     field->type = ID3_FIELD_TYPE_STRINGFULL;
   } else {
+    /* Get the latin-1 string, convert it to a Unicode string */
     field = id3_frame_field (frame, 1);
     field->type = ID3_FIELD_TYPE_STRINGLIST;
   }
 
-  /* Use the specified text encoding */
-  id3_field_settextencoding(field, encoding);
-
   if (frame_name == ID3_FRAME_GENRE) {
     id3_ucs4_t *tmp_ucs4 = id3_utf8_ucs4duplicate ((id3_utf8_t *)data);
     int index = id3_genre_number (tmp_ucs4);
+
     if (index != -1) {
       /* valid genre -- simply store the genre number */
       gchar *tmp = g_strdup_printf("%d", index);
@@ -418,6 +334,7 @@ setFrameText(struct id3_tag *tag,
 	tmp_ucs4 = NULL;
       }
     }
+    // Perhaps not good that this will be NULL sometimes...
     g_free (tmp_ucs4);
   } else {
     if (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1) {
@@ -433,10 +350,16 @@ setFrameText(struct id3_tag *tag,
     }
   }
 
-  if (frame_name == ID3_FRAME_COMMENT)
+  if (gnomad_debug != 0) {
+    g_print("UCS 4 copy of tag %s:\n", frame_name);
+    hexdump((unsigned char *) ucs4, g_utf8_strlen(data,-1)*4);
+  }
+    
+  if (frame_name == ID3_FRAME_COMMENT) {
     res = id3_field_setfullstring (field, ucs4);
-  else
+  } else {
     res = id3_field_setstrings (field, 1, &ucs4);
+  }
 
   g_free (ucs4);
 
@@ -634,7 +557,47 @@ getSonglen (struct id3_tag *tag)
 static gchar*
 getTracknum(struct id3_tag *tag)
 {
-  return getFrameText(tag, "TRCK");
+  gchar trackno[40];
+  gchar *trackstr = getFrameText(tag, "TRCK");
+  gchar *posstr = getFrameText(tag, "TPOS");
+  gint i;
+
+  if (trackstr == NULL) {
+    return NULL;
+  }
+  trackno[0] = '\0';
+  // Take care of any a/b formats
+  for (i = 0; i < strlen(trackstr); i++) {
+    if (trackstr[i] == '/') {
+      // Terminate it at switch character
+      trackstr[i] = '\0';
+      break;
+    }
+  }
+  // "Part of set" variable
+  if (posstr != NULL) {
+    // Same a/b format problem again
+    for (i = 0; i < strlen(posstr); i++) {
+      if (posstr[i] == '/') {
+	// Terminate it at switch character
+	posstr[i] = '\0';
+	break;
+      }
+    }
+    strncpy(trackno, posstr, sizeof(trackno));
+    if (strlen(trackstr) == 1) {
+      strcat(trackno, "0");
+    }
+    strncat(trackno, trackstr, sizeof(trackno));
+    g_free(trackstr);
+    g_free(posstr);
+  } else {
+    strncpy(trackno, trackstr, sizeof(trackno));
+    g_free(trackstr);
+  }
+  // Terminate and return
+  trackno[sizeof(trackno)-1] = '\0';
+  return g_strdup(trackno);
 }
 
 static gchar*
@@ -643,6 +606,15 @@ getOrigFilename(struct id3_tag *tag)
   return getFrameText(tag, "TOFN");
 }
 
+/**
+ * Removes a tag from an MP3 file or adds a tag (and removes any old at the same time)
+ * @param add whether to add a tag (TRUE) or remove a tag (FALSE)
+ * @param path the UTF-8 path to the file, always UTF-8
+ * @param v2tag the ID3v2 tag
+ * @param v2taglen the length (in bytes) of the ID3v2 tag
+ * @param v1tag the ID3v1 tag
+ * @param v1taglen the ID3v1 tag length in bytes (128 bytes)
+ */
 static void
 remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guchar *v1tag, guint v1taglen)
 {
@@ -654,20 +626,35 @@ remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guch
   guint filelength = 0;
   register gchar *buffer;
   guint bufsiz;
-  gboolean retval = FALSE; // not used yet
+  gchar *tmppath;
 
-  // template = g_build_filename(g_get_tmp_dir(), "gnomadXXXXXX"); //FEL
   strcpy(template, g_get_tmp_dir());
   template[strlen(template)+1] = '\0';
   template[strlen(template)] = G_DIR_SEPARATOR;
   strcat(template, "gnomadXXXXXX");
 
   if (gnomad_debug != 0) {
-    g_print("Removing or adding ID3 tags from/to %s\n", path);
+    g_print("Removing ID3 tags from/to %s\n", path);
   }
 
-  f1 = (gint) open(path, O_RDONLY, 0);
+  // Will be UTF-8 on Windows
+  // The following code will support wchar_t filenames by way of glib.
+  tmppath = filename_fromutf8(path);
+  // Sanity check
+  if (tmppath == NULL) {
+    g_print("Path was NULL when calling remove_or_add_tag()\n");
+    return;
+  }
+
+#if GLIB_CHECK_VERSION(2,6,0)  
+  f1 = (gint) g_open(tmppath, READONLY_FLAGS, 0);
+#else
+  f1 = (gint) open(tmppath, READONLY_FLAGS, 0);  
+#endif
+
   if (f1 < 0) {
+    g_print("Could not open file f1 in remove_or_add_tag()\n");
+    g_free(tmppath);
     return;
   }
   // g_print("Opened file f1...\n");
@@ -690,72 +677,37 @@ remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guch
 
     // g_print("Allocated a buffer of %d bytes...\n", bufsiz);
 
-    // g_print("Looking for ID3v2 header tag\n");
+    if (gnomad_debug != 0) {
+      g_print("Looking for ID3v2 header tag\n");
+    }
 
     n = read(f1,tag,10);
     if (n == 10 &&
 	tag[0] == 'I' &&
 	tag[1] == 'D' &&
-	tag[2] == '3') {
-      // g_print("Found ID3v2 tag header...");
-      // Get tag length from the tag
-      header_taglength = (tag[6] << 24) + (tag[7] << 16) + (tag[8] << 8) + tag[9];
-      // This needs some heuristics... first scan until only zeroes are found.
-      // g_print(" given size %d (%08x)\n", header_taglength, header_taglength);
-      n = read(f1,buffer,bufsiz);
-      if (n > 4) {
-	guint m = 0;
-	gboolean foundzeros = FALSE;
-	gboolean founddata = FALSE;
-	
-	// Look for 8 consecutive zeroes
-	while (!foundzeros && (m < n)) {
-	  if (buffer[m] == 0x00 &&
-	      buffer[m+1] == 0x00 &&
-	      buffer[m+2] == 0x00 &&
-	      buffer[m+3] == 0x00 &&
-	      buffer[m+4] == 0x00 &&
-	      buffer[m+5] == 0x00 &&
-	      buffer[m+6] == 0x00 &&
-	      buffer[m+7] == 0x00) {
-	    foundzeros = TRUE;
-	  }
-	  m += 8;
-	}
-	if (foundzeros == TRUE) {
-	  // g_print("Found zeros\n");
-	}
-
-	// Then look for first non-zero
-	while (!founddata && (m < n)) {
-	  if (buffer[m] == '3' &&
-	      buffer[m+1] == 'D' &&
-	      buffer[m+2] == 'I') {
-	    // We found a footer, wind past it and say that we found data.
-	    m += 10;
-	    founddata = TRUE;
-	  } else if (buffer[m] != 0x00) {
-	    founddata = TRUE;
-	    // Tag is now m bytes long!
-	  } else {
-	    m++;
-	  }
-	}
-
-	// If length was properly detected, we now know the length.
-	if (founddata) {
-	  // g_print("Found data\n");
-	  if (header_taglength != m) {
-	    g_print("remove_tag_from_mp3file(): Bad header tag! Given length: %d, detected length: %d\n", header_taglength, m);
-	  }
-	  // Adjust header taglength
-	  header_taglength = m;
-	  // Wind past any RIFF header too
-	  header_taglength += riff_header_size(&buffer[m], n-m);
-	}
-	// At last, include the header too.
+	tag[2] == '3'
+	) {
+      if (gnomad_debug != 0) {
+	g_print("Found ID3v2 tag header...");
+      }
+      // Get tag length from the tag - notice that this is an
+      // UNSYNCED integer, so the highest bit of each 8-bit group 
+      // is unused (always 0).
+      header_taglength = ((tag[6] & 0x7F) << 21) | ((tag[7] & 0x7F) << 14) | ((tag[8] & 0x7F) << 7) | (tag[9] & 0x7F);
+      if ((tag[5] & 0x10) == 0) {
 	header_taglength += 10;
-	// g_print("ID3v2 header (and any RIFF header) %d bytes\n", header_taglength);
+      } else {
+	header_taglength += 20;
+      }
+      if (gnomad_debug != 0) {
+	g_print(" %d (0x%x) bytes\n", header_taglength, header_taglength);
+      }
+
+      // Wind past any RIFF header too
+      n = read(f1,buffer,bufsiz);
+      header_taglength += riff_header_size(&buffer[header_taglength], n);
+      if (gnomad_debug != 0) {
+	g_print("ID3v2 header (and any RIFF header) %d (0x%x) bytes\n", header_taglength, header_taglength);
       }
     } else {
       // Any plain RIFF header is removed too.
@@ -770,7 +722,9 @@ remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guch
     
     if (filelength > 0) {
       // Then detect the length of any ID3v1 tag
-      // g_print("Detecting ID3v1 tag\n");
+      if (gnomad_debug != 0) {
+	g_print("Detecting ID3v1 tag\n");
+      }
       if (lseek(f1, -128, SEEK_END) > 0) {
 	guchar tag[3];
 	gint n;
@@ -780,13 +734,17 @@ remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guch
 	    tag[0] == 'T' &&
 	    tag[1] == 'A' &&
 	    tag[2] == 'G') {
-	  // g_print("Found ID3v1 tag footer...\n");
+	  if (gnomad_debug != 0) {
+	    g_print("Found ID3v1 tag footer, 128 (0x80) bytes...\n");
+	  }
 	  footer_taglength = 128;
 	}
       }
       
       // Then detect the length of any ID3v2 footer tag
-      // g_print("Detecting ID3v2 footer tag\n");
+      if (gnomad_debug != 0) {
+	g_print("Detecting ID3v2 footer tag\n");
+      }
       if (lseek(f1, -10-footer_taglength, SEEK_END) > 0) {
 	guchar tag[10];
 	guint footlen;
@@ -797,42 +755,64 @@ remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guch
 	    tag[0] == '3' &&
 	    tag[1] == 'D' &&
 	    tag[2] == 'I') {
-	  // g_print("Found ID3v2 footer tag\n");
-	  footlen = (tag[6] << 24) + (tag[7] << 16) + (tag[8] << 8) + tag[9];
+	  if (gnomad_debug != 0) {
+	    g_print("Found ID3v2 footer tag...");
+	  }
+	  footlen = ((tag[6] & 0x7F) << 21) | ((tag[7] & 0x7F) << 14) | ((tag[8] & 0x7F) << 7) | (tag[9] & 0x7F);
 	  // First remove the tag headers
 	  footer_taglength += 20;
 	  // Then remove the indicated length (no looking for bad tag here)
 	  footer_taglength += footlen;
+	  if (gnomad_debug != 0) {
+	    g_print(" %d (0x%x) bytes.\n", footlen, footlen);
+	  }
 	}
       }
 
-      // g_print("Header %d bytes, footer %d bytes to be removed.\n", header_taglength, footer_taglength);
+      if (gnomad_debug != 0) {
+	g_print("Header %d (0x%x) bytes, footer %d (0x%x) bytes to be removed.\n", header_taglength, 
+		header_taglength, footer_taglength, footer_taglength);
+      }
 
       if (add) {
 	// Write ID3v2 tag at the beginning of the file
-	write(f2,v2tag,v2taglen);
+	if (gnomad_debug != 0) {
+	  g_print("Adding ID3v2 tag to file\n");
+	}
+	if (v2taglen != write(f2,v2tag,v2taglen)) {
+	  g_print("Error writing ID3v2 header tag in remove_or_add_tag()\n");
+	}
       }
 
       // Next skip past the header, and copy until we reach the footer
+      if (gnomad_debug != 0) {
+	g_print("Copying original file, ");
+      }
       if (lseek(f1, header_taglength, SEEK_SET) >= 0) {
 	guint remain = filelength - header_taglength - footer_taglength;
-	
-	while (remain) {
-	  register guint n;
+
+	if (gnomad_debug != 0) {
+	  g_print("%d bytes.\n", remain);
+	}
+	while (remain > 0) {
+	  register gint n;
 	  
 	  if (remain > bufsiz) {
 	    n = read(f1,buffer,bufsiz);
 	  } else {
 	    n = read(f1,buffer,remain);
 	  }
-	  if (n == -1)
-	    break;
-	  if (n == 0) {
-	    retval = TRUE;
+	  if (n == -1) {
+	    g_print("Error reading source file during file copying in remove_or_add_tag()\n");
 	    break;
 	  }
-	  if (n != write(f2,buffer,n))
+	  if (n == 0) {
 	    break;
+	  }
+	  if (n != write(f2,buffer,n)) {
+	    g_print("Error writing target file during file copying in remove_or_add_tag()\n");
+	    break;
+	  }
 	  remain -= n;
 	}
       }
@@ -841,43 +821,59 @@ remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guch
 
     if (add) {
       // Write ID3v1 tag at the end of the file
-      write(f2,v1tag,v1taglen);
+      if (gnomad_debug != 0) {
+	g_print("Adding ID3v1 tag to file\n");
+      }
+      if (v1taglen != write(f2,v1tag,v1taglen)) {
+	g_print("Error writing ID3v1 footer tag in remove_or_add_tag()\n");
+      }
     }
 
     // Then copy back the stripped file
     // Rewind the temporary file
     lseek(f2, 0, SEEK_SET);
-    // g_print("Copying the file back...\n");
-    f3 = (gint) creat(path, (mode_t) CREATE_FILEMODE);
+    if (gnomad_debug != 0) {
+      g_print("Copying the file back...\n");
+    }
+#if GLIB_CHECK_VERSION(2,8,0)  
+    f3 = (gint) g_creat(tmppath, (mode_t) CREATE_FILEMODE);
+#else
+    f3 = (gint) creat(tmppath, (mode_t) CREATE_FILEMODE);
+#endif
     if (f3) {
       // g_print("Creat() on original file succeeded...\n");
       while (1) {
-	register guint n;
+	register gint n;
 	
 	n = read(f2,buffer,bufsiz);
-	if (n == -1)
-	  break;
-	if (n == 0) {
-	  retval = TRUE;
+	if (n == -1) {
+	  g_print("Error reading source file during final copying in remove_or_add_tag()\n");
 	  break;
 	}
-	if (n != write(f3,buffer,n))
+	if (n == 0) {
 	  break;
+	}
+	if (n != write(f3,buffer,n)) {
+	  g_print("Error writing target file during final copying in remove_or_add_tag()\n");
+	  break;
+	}
       }
       close(f3);
     } else {
-      g_print("Error copying file back!\n");
-      retval = FALSE;
+      g_print("Erroneous file descriptor when copying file back in remove_or_add_tag()\n");
     }
     close(f2);
     // g_print("Deleting %s\n", template);
+#if GLIB_CHECK_VERSION(2,6,0)  
+    g_unlink(template);
+#else
     unlink(template);
+#endif
   } else {
     // In case we couldn't open f2
-    retval = FALSE;
     close(f1);
   }
-
+  g_free(tmppath);
   g_free(buffer);
 }
 
@@ -902,9 +898,21 @@ get_tag_for_mp3file (metadata_t *meta)
   struct id3_tag *tag;
   gchar *tracknum;
   gchar *tmp;
-  gchar *tmppath = filename_fromutf8(meta->path);
+  gchar *tmppath;
   gint songlen;
 
+#ifndef G_OS_WIN32
+  tmppath = filename_fromutf8(meta->path);
+#else
+  // The libid3tag library cannot handle UTF-8/unicode filenames
+  // so under Windows we have to switch the path into the locale.
+  tmppath = g_locale_from_utf8(meta->path,-1,NULL,NULL,NULL);
+#endif
+  if (tmppath == NULL) {
+    g_printf("get_tag_for_mp3file(): filepath was NULL.\n");
+    return;
+  }
+
   /* g_print("Getting tag info for %s...\n", path); */
   fh = id3_file_open(tmppath, ID3_FILE_MODE_READONLY);
   if (fh == 0) {
@@ -942,29 +950,36 @@ get_tag_for_mp3file (metadata_t *meta)
   id3_file_close(fh);
 }
 
-/*
- * TODO: change this function to always strip the current tag
- * and add a new one by rendering both v1 and v2 tags to a buffer
- * and concatenate with the file.
- */
 void
 set_tag_for_mp3file (metadata_t *meta, gboolean override)
 {
   struct id3_file *fh;
   struct id3_tag *tag;
-  gchar *tmppath = filename_fromutf8(meta->path);
+  gchar *tmppath;
   enum id3_field_textencoding encoding;
   id3_length_t tagv2len;
   id3_length_t tagv1len;
   id3_byte_t *tagv2;
   id3_byte_t *tagv1;  
+
+#ifndef G_OS_WIN32
+  tmppath = filename_fromutf8(meta->path);
+#else
+  // The libid3tag library cannot handle UTF-8/unicode filenames
+  // so under Windows we have to switch the path into the locale.
+  tmppath = g_locale_from_utf8(meta->path,-1,NULL,NULL,NULL);
+#endif
+  if (tmppath == NULL) {
+    g_printf("set_tag_for_mp3file(): filepath was NULL.\n");
+    return;
+  }
   
   if (gnomad_debug != 0) {
     g_print("Setting tag info for %s...\n", tmppath);
   }
 
   /* Get the tag for the old file */
-  fh = id3_file_open(tmppath, ID3_FILE_MODE_READWRITE);
+  fh = id3_file_open(tmppath, ID3_FILE_MODE_READONLY);
 
   if (!fh) {
     g_print("Could not open file %s!\n", meta->path);
@@ -980,8 +995,9 @@ set_tag_for_mp3file (metadata_t *meta, gboolean override)
   encoding = get_encoding_of_tag(tag);
   /* ...unless it's ISO_8859_1 and prefs say we should use
      unicode (i.e. ID3v2.4) */
-  if (get_prefs_useid3_unicode() && (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1))
+  if (get_prefs_useid3_unicode() && (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1)) {
     encoding = ID3_FIELD_TEXTENCODING_UTF_8;
+  }
 
   /* Close old file. */
   // id3_file_close(fh);
@@ -1028,7 +1044,7 @@ set_tag_for_mp3file (metadata_t *meta, gboolean override)
   id3_tag_options(tag, ID3_TAG_OPTION_ID3V1, 0);
   tagv2len = id3_tag_render(tag, tagv2);
   if (gnomad_debug != 0) {
-    g_print("Rendered ID3v2 tag:\n");
+    g_print("Rendered ID3v2 tag, length %d (0x%x) bytes:\n", tagv2len, tagv2len);
     hexdump(tagv2, tagv2len);
   }
 
@@ -1036,7 +1052,7 @@ set_tag_for_mp3file (metadata_t *meta, gboolean override)
   id3_tag_options(tag, ID3_TAG_OPTION_ID3V1, ~0);
   tagv1len = id3_tag_render(tag, tagv1);
   if (gnomad_debug != 0) {
-    g_print("Rendered ID3v1 tag:\n");
+    g_print("Rendered ID3v1 tag 128 (0x80) bytes:\n");
     hexdump(tagv1, tagv1len);
   }
 
@@ -1044,7 +1060,7 @@ set_tag_for_mp3file (metadata_t *meta, gboolean override)
   id3_file_close(fh);
 
   /* Totally revamp file */
-  remove_or_add_tag(TRUE, tmppath, tagv2, tagv2len, tagv1, tagv1len);
+  remove_or_add_tag(TRUE, meta->path, tagv2, tagv2len, tagv1, tagv1len);
 
   g_free(tagv2);
   g_free(tagv1);
