@@ -86,6 +86,73 @@ static GSList *playlistlist = NULL;
  */
 static GSList *datafilelist = NULL;
 
+#ifdef HAVE_LIBMTP
+/*
+ * MTP FILETYPE <-> INTERNAL CODEC STRING relation
+ */
+typedef struct {
+  LIBMTP_filetype_t  filetype;
+  const gchar       *codec;
+} mtp_filetype_description_t;
+
+static const mtp_filetype_description_t mtp_filetype_descriptions[] = {
+  { LIBMTP_FILETYPE_WAV,     "WAV" },
+  { LIBMTP_FILETYPE_MP3,     "MP3" },
+  { LIBMTP_FILETYPE_WMA,     "WMA" },
+  { LIBMTP_FILETYPE_OGG,     "OGG" },
+  { LIBMTP_FILETYPE_MP4,     "MP4" },
+  { LIBMTP_FILETYPE_WMV,     "WMV" },
+  { LIBMTP_FILETYPE_AVI,     "AVI" },
+  { LIBMTP_FILETYPE_MPEG,    "MPG" },
+  { LIBMTP_FILETYPE_MPEG,    "MPEG" },
+  { LIBMTP_FILETYPE_ASF,     "ASF" },
+  { LIBMTP_FILETYPE_QT,      "MOV" },
+  { LIBMTP_FILETYPE_JPEG,    "JPG" },
+  { LIBMTP_FILETYPE_JPEG,    "JPEG" },
+  { LIBMTP_FILETYPE_JFIF,    "JFIF" },
+  { LIBMTP_FILETYPE_TIFF,    "TIF" },
+  { LIBMTP_FILETYPE_TIFF,    "TIFF" },
+  { LIBMTP_FILETYPE_BMP,     "BMP" },
+  { LIBMTP_FILETYPE_GIF,     "GIF" }, 
+  { LIBMTP_FILETYPE_PICT,    "PIC" },
+  { LIBMTP_FILETYPE_PICT,    "PICT" },
+  { LIBMTP_FILETYPE_PNG,     "PNG" },
+  { LIBMTP_FILETYPE_WINDOWSIMAGEFORMAT, "WMF" },
+  { LIBMTP_FILETYPE_VCALENDAR2, "ICS" },
+  { LIBMTP_FILETYPE_WINEXEC, "EXE" },
+  { LIBMTP_FILETYPE_WINEXEC, "COM" },
+  { LIBMTP_FILETYPE_WINEXEC, "BAT" },
+  { LIBMTP_FILETYPE_WINEXEC, "DLL" },
+  { LIBMTP_FILETYPE_WINEXEC, "SYS" },
+  { LIBMTP_FILETYPE_UNKNOWN, "N/A" }
+};
+
+static const mtp_filetype_description_t*
+get_mtp_filetype_description(LIBMTP_filetype_t type) {
+  const mtp_filetype_description_t *current;
+
+  current = &mtp_filetype_descriptions[0];
+  while (current->filetype != LIBMTP_FILETYPE_UNKNOWN) {
+    if (current->filetype == type) return current;
+    ++current;
+  }
+  return current; /* I.E. type == LIBMTP_FILETYPE_UNKNOWN */
+}
+
+static const mtp_filetype_description_t*
+get_mtp_filetype_description_by_codec(const gchar *name) {
+  const mtp_filetype_description_t *current;
+
+  g_assert(name != NULL);
+  current = &mtp_filetype_descriptions[0];
+  while (current->filetype != LIBMTP_FILETYPE_UNKNOWN) {
+    if (strcmp(current->codec, name) == 0)
+      return current;
+    ++current;
+  }
+  return current; /* I.E. type == LIBMTP_FILETYPE_UNKNOWN */  
+}
+#endif
 
 /***********************************************************************************/
 /* Public and private functions                                                    */
@@ -759,7 +826,7 @@ gboolean jukebox_select(gint i) {
 
   // Else try to connect to the MTP device
   else if (MTP_device_count > 0) {
-    g_print("This is an MTP device\n");
+    // g_print("This is an MTP device\n");
     use_mtp = TRUE;
     // This is implicitly connected to in the first library call.
     // Set njb to NULL to be certain this is the only device used.
@@ -807,6 +874,11 @@ gboolean jukebox_selected(void)
   return TRUE;
 }
 
+gboolean jukebox_is_mtp(void)
+{
+  return use_mtp;
+}
+
 /* Called at end of session to release jukebox */
 void jukebox_release(void)
 {
@@ -832,24 +904,72 @@ void jukebox_release(void)
  * drop down is then built from */
 static void build_playlist_list()
 {
+  // Make sure we destroyed the old playlist list
+  if (playlistlist != NULL) {
+    g_print("Playlistlist was not NULL!\n");
+  }
+
+  // Destroy the playlist list used for choosing 
+  // during transfer.
+  if (jukebox_playlist != NULL) {
+    GList *tmplist = jukebox_playlist;
+    
+    while(tmplist) {
+      if (tmplist->data != NULL)
+	g_free(tmplist->data);
+      tmplist = tmplist->next;
+      tmplist = tmplist->next;
+    }
+    g_list_free(jukebox_playlist);
+    jukebox_playlist = NULL;
+  }
+
   if (use_mtp) {
-    // Not implemented
-    return;
+#ifdef HAVE_LIBMTP
+    LIBMTP_playlist_t *playlists;
+    
+    // Get playlist listing.
+    playlists = LIBMTP_Get_Playlist_List(mtpdevice);
+    if (playlists != NULL) {
+      LIBMTP_playlist_t *pl, *tmpl;
+      
+      pl = playlists;
+      while (pl != NULL) {
+	gnomadplaylist_entry_t *entry;
+	gchar *tmp;
+	gint i;
+
+	// List of choices when transfering tracks
+	jukebox_playlists++;
+	jukebox_playlist = g_list_append(jukebox_playlist, g_strdup(pl->name));
+	jukebox_playlist = g_list_append(jukebox_playlist, GUINT_TO_POINTER(pl->playlist_id));
+
+	// Widget playlist tree
+	entry = (gnomadplaylist_entry_t *) g_malloc(sizeof(gnomadplaylist_entry_t));
+	entry->name = NULL;
+	entry->plid = 0;
+	entry->tracklist = NULL;
+	entry->name = g_strdup(pl->name);
+	tmp = g_strdup_printf("%lu", pl->playlist_id);
+	entry->plid = tmp;
+
+	// Then build the tree structure immediately
+	for (i = 0; i < pl->no_tracks; i++) {
+	  entry->tracklist = g_slist_append(entry->tracklist,
+					    GUINT_TO_POINTER(pl->tracks[i]));
+	}
+	playlistlist = g_slist_append(playlistlist, (gpointer) entry);
+	if (cancel_jukebox_operation)
+	  break;
+	tmpl = pl;
+	pl = pl->next;
+	LIBMTP_destroy_playlist_t(tmpl);
+      }
+    }
+#endif
   } else {
     njb_playlist_t *playlist;
     
-    if (jukebox_playlist != NULL) {
-      GList *tmplist = jukebox_playlist;
-      
-      while(tmplist) {
-	if (tmplist->data != NULL)
-	  g_free(tmplist->data);
-	tmplist = tmplist->next;
-	tmplist = tmplist->next;
-      }
-      g_list_free(jukebox_playlist);
-      jukebox_playlist = NULL;
-    }
     NJB_Reset_Get_Playlist(pdedevice);
     while (playlist = NJB_Get_Playlist(pdedevice)) {
       jukebox_playlists++;
@@ -865,12 +985,13 @@ static void build_playlist_list()
 static void create_tree_structure()
 {
   if (use_mtp) {
-    // Not implemented yet.
-    playlistlist = NULL;
+    // Implemented as part of build_playlist_list()
     return;
   } else {
     njb_playlist_t *playlist;
-    
+
+    /* Set this too NULL to be certain */
+    playlistlist = NULL;
     /* This is for the actual playlist tree scan */
     NJB_Reset_Get_Playlist(pdedevice);
     while (playlist = NJB_Get_Playlist(pdedevice)) {
@@ -975,6 +1096,7 @@ static void destroy_tree_structure()
     tmplist = tmplist->next;
   }
   g_slist_free(playlistlist);
+  playlistlist = NULL;
 }
 
 
@@ -983,8 +1105,6 @@ static void build_playlist_tree(GtkTreeStore *pltreestore,
 {
   /* First build the structure for dialog boxes */
   build_playlist_list();
-  /* Set this too NULL to be certain */
-  playlistlist = NULL;
   /* Next get the playlists and their respective tracks
    * from the jukebox */
   create_tree_structure();
@@ -1000,17 +1120,69 @@ static void build_playlist_tree(GtkTreeStore *pltreestore,
 static void add_to_playlist(guint playlistid, GList *metalist, 
 			    GtkTreeStore *pltreestore, gboolean threaded)
 {
-  if (use_mtp) {
-    // Not implemented yet.
+  // Sanity check
+  if (!metalist)
     return;
+
+  if (use_mtp) {
+#ifdef HAVE_LIBMTP
+    LIBMTP_playlist_t *pl;
+    GList *tmplist;
+    guint i;
+    guint toadd;
+    uint32_t *copy;
+    
+    pl = LIBMTP_Get_Playlist(mtpdevice, playlistid);
+    if (pl == NULL) {
+      return;
+    }
+
+    // Count the tracks and allocate space for playlist
+    tmplist = metalist;
+    toadd = 0;
+    while(tmplist) {
+      toadd++;
+      tmplist = g_list_next(tmplist);
+    }
+    
+    // Allocate the new playlist
+    copy = g_malloc((pl->no_tracks + toadd) * sizeof(uint32_t));
+    
+    // Copy old tracks
+    for (i = 0; i < pl->no_tracks; i++) {
+      copy[i] = pl->tracks[i];
+    }
+
+    // Remove old contents
+    if (pl->tracks != NULL) {
+      g_free(pl->tracks);
+    }
+    pl->tracks = copy;
+    pl->no_tracks = pl->no_tracks + toadd;
+
+    // Add each new track
+    tmplist = metalist;
+    while(tmplist) {
+      metadata_t *meta;
+      uint32_t id;
+      
+      meta = (metadata_t *) tmplist->data;
+      id = string_to_guint(meta->path);
+      /* g_print("Adding track %lu to playlist %lu\n", id, playlistid); */
+      pl->tracks[i] = id;
+      tmplist = g_list_next(tmplist);
+      i++;
+    }
+    if (LIBMTP_Update_Playlist(mtpdevice, pl) != 0) {
+      g_print("Failed to add tracks to playlist on MTP device.\n");
+    }
+#endif
   } else {
     njb_playlist_track_t *pl_track;
     njb_playlist_t *playlist;
     GList *tmplist = metalist;
     gboolean found = FALSE;
     
-    if (!metalist)
-      return;
     NJB_Reset_Get_Playlist(pdedevice);
     while (playlist = NJB_Get_Playlist(pdedevice)) {
       /* g_print("Is it %lu?\n", playlist->plid); */
@@ -1044,9 +1216,9 @@ static void add_to_playlist(guint playlistid, GList *metalist,
 	NJB_Error_Dump(pdedevice, stderr);
       NJB_Playlist_Destroy(playlist);
     }
-    /* Rebuild the tree afterwards */
-    build_playlist_tree(pltreestore, threaded);
   }
+  /* Rebuild the tree afterwards */
+  build_playlist_tree(pltreestore, threaded);
 }
 
 /* Update the usage information */
@@ -1251,6 +1423,8 @@ gpointer scan_thread(gpointer thread_args)
 #ifdef HAVE_LIBMTP
   LIBMTP_track_t *mtptracks;
   LIBMTP_track_t *mtptrack;
+  LIBMTP_file_t *mtpfiles;
+  LIBMTP_file_t *mtpfile;
 #endif
 #ifndef G_OS_WIN32
   gchar *scan_songs = _("Scanning songs...");
@@ -1281,8 +1455,7 @@ gpointer scan_thread(gpointer thread_args)
     g_free(jukebox_ownerstring);
   if (use_mtp) {
 #ifdef HAVE_LIBMTP
-    jukebox_ownerstring = LIBMTP_Get_Ownername(mtpdevice);
-    g_print("Owner name: %s\n", jukebox_ownerstring);
+    jukebox_ownerstring = LIBMTP_Get_Friendlyname(mtpdevice);
 #else
     jukebox_ownerstring = NULL;
 #endif
@@ -1348,6 +1521,10 @@ gpointer scan_thread(gpointer thread_args)
 #ifdef HAVE_LIBMTP
       LIBMTP_track_t *tmp;
       gchar *tmpyear;
+      const mtp_filetype_description_t *filetype_d;
+
+      // Retrieve filetype infos.
+      filetype_d = get_mtp_filetype_description(mtptrack->filetype);
 
       // Add to metadata holder
       meta->path = g_strdup_printf("%lu", mtptrack->item_id);
@@ -1357,19 +1534,7 @@ gpointer scan_thread(gpointer thread_args)
       meta->album = g_strdup(mtptrack->album);
       meta->length = seconds_to_mmss((guint) (mtptrack->duration/1000));
       meta->size = (guint32) mtptrack->filesize;
-      switch (mtptrack->codec) {
-      case LIBMTP_CODEC_WAV:
-	meta->codec = g_strdup("WAV");
-	break;
-      case LIBMTP_CODEC_MP3:
-	meta->codec = g_strdup("MP3");
-	break;
-      case LIBMTP_CODEC_WMA:
-	meta->codec = g_strdup("WMA");
-	break;
-      default:
-	meta->codec = g_strdup("N/A");
-      }
+      meta->codec = g_strdup(filetype_d->codec);
       meta->trackno = (guint) mtptrack->tracknumber;
       // TODO: Complicated. Fix later.
       tmpyear = g_strdup(mtptrack->date);
@@ -1472,45 +1637,76 @@ gpointer scan_thread(gpointer thread_args)
   jblist_from_songhash(TRUE);
 #endif
 
-  // TODO: when libmtp can do this, rewrite this to support datafile
-  // scanning in a nice way.
-  if (use_mtp) {
-    // Not yet implemented
-  } else {
-    /* Fill in the list of data files */
-    NJB_Reset_Get_Datafile_Tag(pdedevice);
-  }
-
 #ifndef G_OS_WIN32
   gdk_threads_enter();
   gtk_label_set_text(GTK_LABEL(args->label), scan_datafiles);
   gdk_threads_leave();
 #endif
 
+  /* Next retrieve the datafile listing */
+  last_item = FALSE;
+  if (use_mtp) {
+#ifdef HAVE_LIBMTP
+    mtpfiles = LIBMTP_Get_Filelisting(mtpdevice);
+    mtpfile = mtpfiles;
+    if (mtpfile == NULL) {
+      last_item = TRUE;
+    }
+#else
+    last_item = TRUE;
+#endif
+  } else {
+    NJB_Reset_Get_Datafile_Tag(pdedevice);
+    datatag = NJB_Get_Datafile_Tag(pdedevice);
+    if (datatag == NULL) {
+      last_item = TRUE;
+    }
+  }
+
   /* Destroy the old datafile list */
   destroy_datafile_list();
 
-  if (use_mtp) {
-    // Not yet implemented
-  } else {
-    while (datatag = NJB_Get_Datafile_Tag (pdedevice)) {
-      metadata_t *meta;
-      u_int64_t filesize;
-      gint i;
-      gchar *scanstring;
-      
-      jukebox_datafiles++;
+
+  while (!last_item) {
+    metadata_t *meta;
+    u_int64_t filesize;
+    gint i;
+    gchar *scanstring;
     
+    jukebox_datafiles++;
+
 #ifndef G_OS_WIN32
-      scanstring = g_strdup_printf(_("%u data files scanned"), jukebox_datafiles);
-      gdk_threads_enter();
-      gtk_label_set_text(GTK_LABEL(args->label), scanstring);
-      gdk_threads_leave();
-      g_free(scanstring);
+    scanstring = g_strdup_printf(_("%u data files scanned"), jukebox_datafiles);
+    gdk_threads_enter();
+    gtk_label_set_text(GTK_LABEL(args->label), scanstring);
+    gdk_threads_leave();
+    g_free(scanstring);
 #endif
 
-      meta = new_metadata_t();
-      /* g_print("Scan: filename %s, size %d\n", datatag->filename, strlen(datatag->filename)); */
+    // Create a structure to hold the data in the columns
+    meta = new_metadata_t();
+
+    if (use_mtp) {
+#ifdef HAVE_LIBMTP
+      LIBMTP_file_t *tmp;
+      // Add to metadata holder
+      meta->path = g_strdup_printf("%lu", mtpfile->item_id);
+      meta->size = (guint32) mtpfile->filesize;
+      meta->filename = g_strdup(mtpfile->filename);
+      // These two are not supported yet.
+      meta->folder = g_strdup("\\");
+      meta->protect = FALSE;
+      
+      tmp = mtpfile;
+      mtpfile = mtpfile->next;
+      LIBMTP_destroy_file_t(tmp);
+      if (mtpfile == NULL) {
+	last_item = TRUE;
+      }
+#else
+      last_item = TRUE;
+#endif
+    } else {
       /* Convert filesize from 64 bit unsigned integer value */
       filesize = (u_int64_t) datatag->filesize;
       /* FIXME: here we loos all the 64-bit quality, because meta->size is 32-bit ... */
@@ -1532,23 +1728,28 @@ gpointer scan_thread(gpointer thread_args)
       }
       /* File ID */
       meta->path = g_strdup_printf("%lu", datatag->dfid);
-      datafilelist = g_slist_append(datafilelist, (gpointer) meta);
       // destroy_metadata_t(meta);
       NJB_Datafile_Destroy (datatag);
-      if (cancel_jukebox_operation)
-	break;
+      datatag = NJB_Get_Datafile_Tag(pdedevice);
+      if (datatag == NULL) {
+	last_item = TRUE;
+      }
     }
-
-    /* Then refill and sort everything */
-#ifndef G_OS_WIN32
-    gdk_threads_enter();
-    rebuild_datafile_list("\\");
-    gtk_label_set_text(GTK_LABEL(args->label), scan_playlists);
-    gdk_threads_leave();
-#endif
+    
+    datafilelist = g_slist_append(datafilelist, (gpointer) meta);
+    if (cancel_jukebox_operation)
+      break;
   }
-
-  // The refresh the playlist tree
+  
+  /* Then refill and sort everything */
+#ifndef G_OS_WIN32
+  gdk_threads_enter();
+  rebuild_datafile_list("\\");
+  gtk_label_set_text(GTK_LABEL(args->label), scan_playlists);
+  gdk_threads_leave();
+#endif
+  
+  // Then refresh the playlist tree
   if (gnomad_debug != 0) {
     g_print("Building playlist tree...\n");
   }
@@ -1595,7 +1796,7 @@ static int pde_progress (u_int64_t sent, u_int64_t total, const char* buf, unsig
  * Same thing but for MTP.
  */
 #ifdef HAVE_LIBMTP
-static LIBMTP_progressfunc_t mtp_progress;
+//static const LIBMTP_progressfunc_t mtp_progress;
 static int mtp_progress (uint64_t const sent, uint64_t const total,
 			 void const * const data)
 {
@@ -1708,6 +1909,7 @@ gpointer jb2hd_thread(gpointer thread_args)
 #endif
   /* Free the memory used by the list */
   destroy_metalist(args->metalist);
+  g_free(args);
 }
 
 
@@ -1742,10 +1944,21 @@ gpointer jb2hd_data_thread(gpointer thread_args)
       // convert filenames into unicode...
       tmpfname = g_locale_from_utf8(jbmeta->filename,-1,NULL,NULL,NULL);
 #endif
-      if (NJB_Get_File(pdedevice, id, jbmeta->size, tmpfname, pde_progress, NULL) == -1) {
-	g_free(tmpfname);
-	NJB_Error_Dump(pdedevice, stderr);
-	goto data_error;
+      if (use_mtp) {
+#ifdef HAVE_LIBMTP
+	int ret;
+	ret = LIBMTP_Get_File_To_File(mtpdevice, id, tmpfname, mtp_progress, NULL);
+	if (ret != 0) {
+	  g_free(tmpfname);
+	  goto data_error;
+	}
+#endif
+      } else {
+	if (NJB_Get_File(pdedevice, id, jbmeta->size, tmpfname, pde_progress, NULL) == -1) {
+	  g_free(tmpfname);
+	  NJB_Error_Dump(pdedevice, stderr);
+	  goto data_error;
+	}
       }
       g_free(tmpfname);
       /* Add to disk listbox and re-sort */
@@ -1769,6 +1982,7 @@ gpointer jb2hd_data_thread(gpointer thread_args)
   gtk_widget_destroy(args->dialog);
   g_idle_add(call_fill_in_dir, (gpointer) HDDATA_LIST);
 #endif
+  g_free(args);
 }
 
 
@@ -1833,6 +2047,10 @@ gpointer hd2jb_thread(gpointer thread_args)
 #ifdef HAVE_LIBMTP
 	LIBMTP_track_t *trackmeta = LIBMTP_new_track_t();
 	int ret;
+	const mtp_filetype_description_t *filetype_d;
+
+	// Retrieve filetype infos
+	filetype_d = get_mtp_filetype_description_by_codec(hdmeta->codec);
 	
 	// This will be filled in, dummy set to 0.
 	trackmeta->item_id = 0;
@@ -1846,19 +2064,12 @@ gpointer hd2jb_thread(gpointer thread_args)
 	// This need to be in milliseconds
 	trackmeta->duration = length * 1000;
 	trackmeta->filesize = get_64bit_file_size(tmpfname);
-	if (!strcmp(hdmeta->codec, "WAV")) {
-	  trackmeta->codec = LIBMTP_CODEC_WAV;
-	} else if (!strcmp(hdmeta->codec, "MP3")) {
-	  trackmeta->codec = LIBMTP_CODEC_MP3;
-	} else if (!strcmp(hdmeta->codec, "WMA")) {
-	  trackmeta->codec = LIBMTP_CODEC_WMA;
-	} else {
-	  trackmeta->codec = LIBMTP_CODEC_UNKNOWN;
-	}
+	trackmeta->filetype = filetype_d->filetype;
 	// TODO: Discarding return value as for now
 	g_print("Transferring MTP track...\n");
+	// Hardcode parent (folder) to 0 (== root directory)
 	ret = LIBMTP_Send_Track_From_File(mtpdevice, tmpfname, trackmeta,
-					  mtp_progress, NULL);
+					  mtp_progress, NULL, 0);
 	if (ret != 0) {
 	  g_print("Error sending file \"%s\" to MTP device!\n", tmpfname);
 	  goto hd2jb_cleanup;
@@ -1992,16 +2203,36 @@ gpointer hd2jb_data_thread(gpointer thread_args)
 	/* This shouldn't happen, but include it anyway. */
 	folder = rootdir;
       }
-      /* g_print("Storing %s on jukebox as %s in folder %s...\n", hdmeta->path, hdmeta->filename, folder); */
+      g_print("Storing %s (extension %s) on jukebox as %s in folder %s...\n", hdmeta->path, hdmeta->codec, hdmeta->filename, folder);
       gdk_threads_enter();
       gtk_label_set_text(GTK_LABEL(args->label), hdmeta->filename);
       gtk_progress_bar_update(GTK_PROGRESS_BAR(progress_bar), (gfloat) 0);
       gdk_threads_leave();
       tmpfname = filename_fromutf8(hdmeta->path);
-      if (NJB_Send_File (pdedevice, tmpfname, hdmeta->filename, folder, pde_progress, NULL, &id) == -1) {
-	g_free(tmpfname);
-	NJB_Error_Dump(pdedevice, stderr);
-	goto hd2jb_data_cleanup;
+      if (use_mtp) {
+#ifdef HAVE_LIBMTP
+	LIBMTP_file_t *filemeta = LIBMTP_new_file_t();
+	const mtp_filetype_description_t *filetype_d;
+
+	// Retrieve filetype infos.
+	filetype_d = get_mtp_filetype_description_by_codec(hdmeta->codec);
+	filemeta->filename = g_strdup(hdmeta->filename);
+	filemeta->filesize = (uint64_t) hdmeta->size;
+	filemeta->filetype = filetype_d->filetype;
+	
+	if (LIBMTP_Send_File_From_File(mtpdevice, tmpfname, filemeta, mtp_progress, NULL, 0) != 0) {
+	  LIBMTP_destroy_file_t(filemeta);
+	  g_free(tmpfname);
+	  goto hd2jb_data_cleanup;
+	}
+	LIBMTP_destroy_file_t(filemeta);
+#endif
+      } else {
+	if (NJB_Send_File (pdedevice, tmpfname, hdmeta->filename, folder, pde_progress, NULL, &id) == -1) {
+	  g_free(tmpfname);
+	  NJB_Error_Dump(pdedevice, stderr);
+	  goto hd2jb_data_cleanup;
+	}
       }
       g_free(tmpfname);
       /* g_print("%s stored on jukebox with ID: %lu\n", hdmeta->filename, id); */
@@ -2041,6 +2272,45 @@ gpointer hd2jb_data_thread(gpointer thread_args)
 }
 
 
+#ifdef HAVE_LIBMTP
+static gboolean delete_one_track_from_playlist(LIBMTP_playlist_t *pl, uint32_t trackid)
+{
+  LIBMTP_playlist_t *tmp;
+  guint i;
+  guint instances = 0;
+
+  // See if the track is on this playlist, remove it if it is
+  for (i = 0; i < pl->no_tracks; i++) {
+    if (pl->tracks[i] == trackid) {
+      instances ++;
+    }
+  }
+
+  // If there were instances of this track on this playlist,
+  // remove them.
+  if (instances != 0) {
+    uint32_t *copy = g_malloc((pl->no_tracks - instances) * sizeof(uint32_t));
+    guint j = 0;
+    
+    for (i = 0; i < pl->no_tracks; i++) {
+      if (pl->tracks[i] != trackid) {
+	copy[j] = pl->tracks[i];
+	j++;
+      }
+    }
+    g_free(pl->tracks);
+    pl->tracks = copy;
+    pl->no_tracks = pl->no_tracks - instances;
+    if (LIBMTP_Update_Playlist(mtpdevice, pl) != 0) {
+      g_printf("Failed to remove tracks from playlist during device update\n");
+      return FALSE;
+    }
+  }
+  return TRUE;
+}
+#endif // HAVE_LIBMTP
+
+
 /* Called after file deletion to remove the deleted
  * tracks from any playlists they may occur in. UNTHREADED */
 static void remove_tracks_from_playlists(GList *metalist,
@@ -2054,8 +2324,9 @@ static void remove_tracks_from_playlists(GList *metalist,
   /* g_print("Called remove_tracks_from_playlists()\n"); */  
   tmplist = metalist;
 
+  // FIXME: this routine is O(n^2) due to two linear
+  //        searches. Should be improved some day.
   while (tmplist) {
-    njb_playlist_t *playlist;
     metadata_t *meta;
     u_int32_t id;  
     
@@ -2063,8 +2334,31 @@ static void remove_tracks_from_playlists(GList *metalist,
     id = string_to_guint(meta->path);
 
     if (use_mtp) {
-      // Not implemented yet
+#ifdef HAVE_LIBMTP
+      LIBMTP_playlist_t *playlists;
+
+      // Get playlist listing.
+      playlists = LIBMTP_Get_Playlist_List(mtpdevice);
+      if (playlists == NULL) {
+	// No playlists == nothing to do
+	return;
+      } else {
+	LIBMTP_playlist_t *pl, *tmp;
+	pl = playlists;
+	while (pl != NULL) {
+	  if(!delete_one_track_from_playlist(pl, id)) {
+	    g_print("Could not remove track %d from playlist %d\n", id, pl->playlist_id);
+	  }
+	  // Then iterate over next playlist
+	  tmp = pl;
+	  pl = pl->next;
+	  LIBMTP_destroy_playlist_t(tmp);
+	}
+      }
+#endif
     } else {
+      njb_playlist_t *playlist;
+
       NJB_Reset_Get_Playlist(pdedevice);
 
       while (playlist = NJB_Get_Playlist(pdedevice)) {
@@ -2103,7 +2397,12 @@ void jukebox_delete_files(GList *metalist)
     /* g_print("Deleting: %u from jukebox library\n", id); */
     if (id) {
       if (use_mtp) {
-	// Not implemented yet
+#ifdef HAVE_LIBMTP
+	if (LIBMTP_Delete_Object(mtpdevice, id) != 0) {
+	  // Error message
+	  g_print("Unable to delete files.\n");
+	}
+#endif
       } else {
 	if ( NJB_Delete_Datafile(pdedevice, id) == -1 ) {
 	  NJB_Error_Dump(pdedevice, stderr);
@@ -2144,9 +2443,9 @@ void jukebox_delete_tracks(GList *metalist,
       
       if (use_mtp) {
 #ifdef HAVE_LIBMTP
-	int ret;
-	// TODO: what do we do if the returncode is != 0?
-	ret = LIBMTP_Delete_Track(mtpdevice, id);
+	if (LIBMTP_Delete_Object(mtpdevice, id) != 0) {
+	  g_print("Unable to delete tracks.\n");
+	}
 #endif
       } else {
 	if ( NJB_Delete_Track(pdedevice, id) == -1 ) {
@@ -2199,7 +2498,11 @@ void jukebox_set_metadata (metadata_t *meta)
 #ifdef HAVE_LIBMTP
     LIBMTP_track_t *trackmeta = LIBMTP_new_track_t();
     int ret;
+    const mtp_filetype_description_t *filetype_d;
 
+    // Retrieve filetype infos.
+    filetype_d = get_mtp_filetype_description_by_codec(meta->codec);
+    
     trackmeta->item_id = id;
     trackmeta->title = g_strdup(meta->title);
     trackmeta->album = g_strdup(meta->album);
@@ -2213,15 +2516,7 @@ void jukebox_set_metadata (metadata_t *meta)
     trackmeta->duration = length * 1000;
     // This cannot be set this way but send it anyway
     trackmeta->filesize = (uint64_t) meta->size;
-    if (!strcmp(meta->codec, "WAV")) {
-      trackmeta->codec = LIBMTP_CODEC_WAV;
-    } else if (!strcmp(meta->codec, "MP3")) {
-      trackmeta->codec = LIBMTP_CODEC_MP3;
-    } else if (!strcmp(meta->codec, "WMA")) {
-      trackmeta->codec = LIBMTP_CODEC_WMA;
-    } else {
-      trackmeta->codec = LIBMTP_CODEC_UNKNOWN;
-    }
+    trackmeta->filetype = filetype_d->filetype;
     // TODO: Discarding return value as for now
     ret = LIBMTP_Update_Track_Metadata(mtpdevice, trackmeta);
     LIBMTP_destroy_track_t(trackmeta);
@@ -2314,26 +2609,39 @@ guint jukebox_create_playlist(gchar *plname, GtkTreeStore *pltreestore)
   guint plid = 0;
 
   /* g_print("Called create playlist\n"); */
-  /* Create the new playlist in memory */
-  playlist = NJB_Playlist_New();
-  if (playlist == NULL) {
-    return 0;
+  if (use_mtp) {
+#ifdef HAVE_LIBMTP
+    LIBMTP_playlist_t *pl;
+
+    pl = LIBMTP_new_playlist_t();
+    pl->name = g_strdup(plname);
+    // This will create the new playlist in the default folder
+    if (LIBMTP_Create_New_Playlist(mtpdevice, pl, 0) != 0) {
+      create_error_dialog(_("Could not create playlist"));
+      return 0;
+    }
+    LIBMTP_destroy_playlist_t(pl);
+#endif
+  } else {
+    /* Create the new playlist in memory */
+    playlist = NJB_Playlist_New();
+    if (playlist == NULL) {
+      return 0;
+    }
+    if (NJB_Playlist_Set_Name(playlist, plname) == -1) {
+      NJB_Error_Dump(pdedevice, stderr);
+      return 0;
+    }
+    jukebox_locked = TRUE;
+    /* g_print("Calling NJB_Update_Playlist\n"); */
+    if (NJB_Update_Playlist(pdedevice, playlist) == -1) {
+      create_error_dialog(_("Could not create playlist"));
+      NJB_Error_Dump(pdedevice, stderr);
+    }
+    jukebox_playlists++;
+    plid = playlist->plid;
+    NJB_Playlist_Destroy(playlist);
   }
-  if (NJB_Playlist_Set_Name(playlist, plname) == -1) {
-    NJB_Error_Dump(pdedevice, stderr);
-    return 0;
-  }
-  jukebox_locked = TRUE;
-  /* g_print("Calling NJB_Update_Playlist\n"); */
-  if (NJB_Update_Playlist(pdedevice, playlist) == -1) {
-    gdk_threads_enter();
-    create_error_dialog(_("Could not create playlist"));
-    gdk_threads_leave();
-    NJB_Error_Dump(pdedevice, stderr);
-  }
-  jukebox_playlists++;
-  plid = playlist->plid;
-  NJB_Playlist_Destroy(playlist);
   build_playlist_tree(pltreestore, FALSE);
   jukebox_locked = FALSE;
   /* g_print("Created playlist\n"); */
@@ -2344,11 +2652,17 @@ guint jukebox_create_playlist(gchar *plname, GtkTreeStore *pltreestore)
 void jukebox_delete_playlist(guint plid)
 {
   jukebox_locked = TRUE;
-  if (NJB_Delete_Playlist(pdedevice, plid) == -1) {
-    gdk_threads_enter();
-    create_error_dialog(_("Could not delete playlist"));
-    gdk_threads_leave();
-    NJB_Error_Dump(pdedevice, stderr);
+  if (use_mtp) {
+#ifdef HAVE_LIBMTP
+    if (LIBMTP_Delete_Object(mtpdevice, plid) != 0) {
+      create_error_dialog(_("Could not delete playlist"));
+    }
+#endif
+  } else {
+    if (NJB_Delete_Playlist(pdedevice, plid) == -1) {
+      create_error_dialog(_("Could not delete playlist"));
+      NJB_Error_Dump(pdedevice, stderr);
+    }
   }
   jukebox_playlists--;
   build_playlist_list();
@@ -2358,44 +2672,65 @@ void jukebox_delete_playlist(guint plid)
 /* Rename the playlist with ID plid UNTHREADED */
 void jukebox_rename_playlist(guint plid, gchar *name, GtkTreeStore *pltreestore, gboolean threaded)
 {
-  njb_playlist_t *playlist;
-  gboolean found = FALSE;
-  
-  jukebox_locked = TRUE;
-  NJB_Reset_Get_Playlist(pdedevice);
-  while (playlist = NJB_Get_Playlist(pdedevice)) {
-    /* g_print("Playlist %u == Playlist %u ... ",playlist->plid, plid); */
-    if (playlist->plid == plid) {
-      /* g_print("Yes.\n"); */
-      found = TRUE;
-      break;
+  if (use_mtp) {
+#ifdef HAVE_LIBMTP
+    // Get metadata for one single playlist, update and write back.
+    LIBMTP_playlist_t *pl;
+
+    pl = LIBMTP_Get_Playlist(mtpdevice, plid);
+    if (pl->name != NULL) {
+      g_free(pl->name);
     }
-    /* g_print("No.\n"); */
-    NJB_Playlist_Destroy(playlist);
-  }
-  if (found) {
-    if (NJB_Playlist_Set_Name(playlist, name) == -1) {
-      NJB_Error_Dump(pdedevice, stderr);
-    } else {
-      if (NJB_Update_Playlist(pdedevice, playlist) == -1) {
-	if (threaded)
-	  gdk_threads_enter();
-	create_error_dialog(_("Could not rename playlist"));
-	if (threaded)
-	  gdk_threads_leave();
-	NJB_Error_Dump(pdedevice, stderr);
-      }
+    pl->name = g_strdup(name);
+    if (LIBMTP_Update_Playlist(mtpdevice, pl) != 0) {
+      if (threaded)
+	gdk_threads_enter();
+      create_error_dialog(_("Could not rename playlist"));
+      if (threaded)
+	gdk_threads_leave();
     }
-    NJB_Playlist_Destroy(playlist);
-    build_playlist_tree(pltreestore, FALSE);
+    LIBMTP_destroy_playlist_t(pl);
+#endif
   } else {
-    if (threaded)
-      gdk_threads_enter();
-    create_error_dialog(_("Could not locate playlist to rename!"));
-    if (threaded)
-      gdk_threads_leave();
+    njb_playlist_t *playlist;
+    gboolean found = FALSE;
+  
+    jukebox_locked = TRUE;
+    NJB_Reset_Get_Playlist(pdedevice);
+    while (playlist = NJB_Get_Playlist(pdedevice)) {
+      /* g_print("Playlist %u == Playlist %u ... ",playlist->plid, plid); */
+      if (playlist->plid == plid) {
+	/* g_print("Yes.\n"); */
+	found = TRUE;
+	break;
+      }
+      /* g_print("No.\n"); */
+      NJB_Playlist_Destroy(playlist);
+    }
+    if (found) {
+      if (NJB_Playlist_Set_Name(playlist, name) == -1) {
+	NJB_Error_Dump(pdedevice, stderr);
+      } else {
+	if (NJB_Update_Playlist(pdedevice, playlist) == -1) {
+	  if (threaded)
+	    gdk_threads_enter();
+	  create_error_dialog(_("Could not rename playlist"));
+	  if (threaded)
+	    gdk_threads_leave();
+	  NJB_Error_Dump(pdedevice, stderr);
+	}
+      }
+      NJB_Playlist_Destroy(playlist);
+      build_playlist_tree(pltreestore, FALSE);
+    } else {
+      if (threaded)
+	gdk_threads_enter();
+      create_error_dialog(_("Could not locate playlist to rename!"));
+      if (threaded)
+	gdk_threads_leave();
+    }
+    jukebox_locked = FALSE;
   }
-  jukebox_locked = FALSE;
 }
 
 /* This is a public caller func for the above UNTHREADED */
@@ -2420,7 +2755,13 @@ void add_tracks_to_playlists(GList *playlists, GList *metalist, GtkTreeStore *pl
 guint jukebox_randomize_playlist(guint plist, GtkTreeStore *pltreestore)
 {
   guint new_plid = plist;
+
   
+  if (use_mtp) {
+    // FIXME: implement for MTP, not yet done.
+    return plist;
+  }
+
   if (plist != 0) {
     njb_playlist_t *playlist;
     gboolean found = FALSE;
@@ -2510,53 +2851,71 @@ guint jukebox_delete_track_from_playlist(guint trackid, guint plist, GtkTreeStor
   guint new_plid = plist;
 
   if (trackid != 0 && plist != 0) {
-    njb_playlist_t *playlist;
-    gboolean found = FALSE;
+
 
     jukebox_locked = TRUE;
-    // g_print("Removing track: %lu from playlist %lu\n", trackid, plist);
-    NJB_Reset_Get_Playlist(pdedevice);
-    while (playlist = NJB_Get_Playlist(pdedevice)) {
-      // g_print("Is it in %lu? ...", playlist->plid);
-      if (playlist->plid == plist) {
-	// g_print("YES!\n");
-	found = TRUE;
-	break;
+    g_print("Removing track: %lu from playlist %lu\n", trackid, plist);
+
+    if (use_mtp) {
+#ifdef HAVE_LIBMTP
+      LIBMTP_playlist_t *pl;
+ 
+      pl = LIBMTP_Get_Playlist(mtpdevice, plist);
+      if (pl == NULL) {
+	g_print("Failed to locate playlist for track removal.\n");
+      } else {
+	if (!delete_one_track_from_playlist(pl, trackid)) {
+	  g_print("Failed to remove track %d from playlist %d\n", trackid, pl->playlist_id);
+	}
+	LIBMTP_destroy_playlist_t(pl);
       }
-      // g_print("NO.\n");
-      NJB_Playlist_Destroy(playlist);
-    }
-    /* If the playlist was found, remove the track */
-    if (found) {
-      njb_playlist_track_t *track;
-      
-      // g_print("Found playlist %lu\n", playlist->plid);
-      while (track = NJB_Playlist_Gettrack(playlist)) {
-	if (trackid == track->trackid) {
-	  /* When the track is located in a playlist, remove it */
-	  // g_print("Removing track: %lu from playlist %lu\n", track->trackid, playlist->plid);
-	  
-	  if (track->prev != NULL)
-	    track->prev->next = track->next;
-	  else
-	    playlist->first = track->next;
-	  if (track->next != NULL)
-	    track->next->prev = track->prev;
-	  NJB_Playlist_Track_Destroy(track);
-	  playlist->ntracks--;
-	  playlist->_state = NJB_PL_CHTRACKS;
+#endif
+    } else {
+      gboolean found = FALSE;
+      njb_playlist_t *playlist;
+      NJB_Reset_Get_Playlist(pdedevice);
+      while (playlist = NJB_Get_Playlist(pdedevice)) {
+	// g_print("Is it in %lu? ...", playlist->plid);
+	if (playlist->plid == plist) {
+	  // g_print("YES!\n");
+	  found = TRUE;
 	  break;
 	}
+	// g_print("NO.\n");
+	NJB_Playlist_Destroy(playlist);
       }
-      if (playlist->_state==NJB_PL_CHTRACKS) {
-	/* g_print("Called NJB_Update_Playlist...\n"); */
-	if (NJB_Update_Playlist(pdedevice, playlist) == -1)
-	  NJB_Error_Dump(pdedevice, stderr);
-	new_plid = playlist->plid;
+      /* If the playlist was found, remove the track */
+      if (found) {
+	njb_playlist_track_t *track;
+	
+	// g_print("Found playlist %lu\n", playlist->plid);
+	while (track = NJB_Playlist_Gettrack(playlist)) {
+	  if (trackid == track->trackid) {
+	    /* When the track is located in a playlist, remove it */
+	    // g_print("Removing track: %lu from playlist %lu\n", track->trackid, playlist->plid);
+	    
+	    if (track->prev != NULL)
+	      track->prev->next = track->next;
+	    else
+	      playlist->first = track->next;
+	    if (track->next != NULL)
+	      track->next->prev = track->prev;
+	    NJB_Playlist_Track_Destroy(track);
+	    playlist->ntracks--;
+	    playlist->_state = NJB_PL_CHTRACKS;
+	    break;
+	  }
+	}
+	if (playlist->_state==NJB_PL_CHTRACKS) {
+	  /* g_print("Called NJB_Update_Playlist...\n"); */
+	  if (NJB_Update_Playlist(pdedevice, playlist) == -1)
+	    NJB_Error_Dump(pdedevice, stderr);
+	  new_plid = playlist->plid;
+	}
+	NJB_Playlist_Destroy(playlist);
+      } else {
+	create_error_dialog(_("Could not find the track in the playlist"));
       }
-      NJB_Playlist_Destroy(playlist);
-    } else {
-      create_error_dialog(_("Could not find the track in the playlist"));
     }
   }
   jukebox_locked = FALSE;
@@ -2577,18 +2936,22 @@ void jukebox_set_ownerstring(gchar *owner)
 {
   jukebox_locked = TRUE;
   if (use_mtp) {
-    // Not implemented yet
+#ifdef HAVE_LIBMTP
+    if (LIBMTP_Set_Friendlyname(mtpdevice, owner) != 0) {
+      create_error_dialog(_("Could not set owner string on the device"));
+    }
+#endif
   } else {
     NJB_Set_Owner_String (pdedevice, owner);
-    if (jukebox_ownerstring != NULL)
-      g_free(jukebox_ownerstring);
-    jukebox_ownerstring = g_strdup(owner);
-    /* The ownerstring max length is 64 bytes
-     * including the null-terminator according
-     * to the protocol specification */
-    if (strlen(jukebox_ownerstring) >= OWNER_STRING_LENGTH)
-      jukebox_ownerstring[OWNER_STRING_LENGTH-1] = '\0';
   }
+  if (jukebox_ownerstring != NULL)
+    g_free(jukebox_ownerstring);
+  jukebox_ownerstring = g_strdup(owner);
+  /* The ownerstring max length is 64 bytes
+   * including the null-terminator according
+   * to the protocol specification */
+  if (strlen(jukebox_ownerstring) >= OWNER_STRING_LENGTH)
+    jukebox_ownerstring[OWNER_STRING_LENGTH-1] = '\0';
   jukebox_locked = FALSE;
 }
 
@@ -2626,7 +2989,9 @@ static void hhmmss (u_int16_t seconds, u_int16_t *hh, u_int16_t *mm, u_int16_t *
 void jukebox_reset_get_eax(void)
 {
   jukebox_locked = TRUE;
-  NJB_Reset_Get_EAX_Type(pdedevice);
+  if (!use_mtp) {
+    NJB_Reset_Get_EAX_Type(pdedevice);
+  }
 }
 
 
@@ -2634,25 +2999,32 @@ void jukebox_reset_get_eax(void)
  * something fails. */
 njb_eax_t *jukebox_get_eax(void)
 {
-  njb_eax_t *eax = NJB_Get_EAX_Type(pdedevice);
+  if (!use_mtp) {
+    njb_eax_t *eax = NJB_Get_EAX_Type(pdedevice);
 
-  if (eax == NULL)
-    jukebox_locked = FALSE;
-  return eax;
+    if (eax == NULL)
+      jukebox_locked = FALSE;
+    return eax;
+  }
+  return NULL;
 }
 
 /* Free the memory used by the EAX structure */
 void jukebox_destroy_eax(njb_eax_t *eax)
 {
-  NJB_Destroy_EAX_Type(eax);
+  if (!use_mtp) {
+    NJB_Destroy_EAX_Type(eax);
+  }
 }
 
 void jukebox_adjust_eax(guint16 effect, guint16 patch, gint16 value)
 {
-  if (created_play_mutex) {
-    g_mutex_lock(play_thread_mutex);
-    NJB_Adjust_EAX(pdedevice, effect, patch, value);
-    g_mutex_unlock(play_thread_mutex);
+  if (!use_mtp) {
+    if (created_play_mutex) {
+      g_mutex_lock(play_thread_mutex);
+      NJB_Adjust_EAX(pdedevice, effect, patch, value);
+      g_mutex_unlock(play_thread_mutex);
+    }
   }
 }
 
@@ -2664,6 +3036,11 @@ GList *jukebox_get_playlist_for_play(guint plid)
   njb_playlist_t *playlist;
   gboolean found = FALSE;
 
+  if (use_mtp) {
+    // FIXME: not implemented for MTP
+    return NULL;
+  }
+
   jukebox_locked = TRUE;
   NJB_Reset_Get_Playlist(pdedevice);
   while (playlist = NJB_Get_Playlist(pdedevice)) {
@@ -2700,6 +3077,11 @@ static void send_playlist(GList *list)
 {
   gboolean first = TRUE;
   GList *tmplist = list;
+
+  if (use_mtp) {
+    // FIXME: not implemented for MTP.
+    return;
+  }
   
   if (tmplist != NULL)
     NJB_Stop_Play(pdedevice);
@@ -2833,6 +3215,10 @@ gpointer play_thread(gpointer thread_args)
   gboolean repeat = TRUE;
   gfloat position;
 
+  /* Just mail out on MTP devices */
+  if (use_mtp) {
+    return;
+  }
   /* This mutex is to avoid collisions with EAX changes */
   if (!created_play_mutex) {
     g_assert (play_thread_mutex == NULL);
@@ -2942,33 +3328,39 @@ gpointer play_thread(gpointer thread_args)
  */
 void jukebox_synchronize_time(void)
 {
-  njb_time_t *jukeboxtime;
   GTimeVal currenttimeval;
   GTime currenttime;
-  struct tm *tm;
-  
-  // Get the time from the jukebox, then modify it.
-  jukeboxtime = NJB_Get_Time(pdedevice);
+
   g_get_current_time(&currenttimeval);
   currenttime = (GTime) currenttimeval.tv_sec;
 
-  // Uses UNIX library calls, might need to be portablized
-  tm = localtime((time_t *) &currenttime);
-  /*
-    g_print("Setting: %d-%d-%d (wkd: %d) %d:%d:%d\n", 1900+tm->tm_year,
-    1+tm->tm_mon, tm->tm_mday, tm->tm_wday-1,
-    tm->tm_hour, tm->tm_min, tm->tm_sec);
-  */
-  jukeboxtime->year = 1900+tm->tm_year;
-  jukeboxtime->month = 1+tm->tm_mon;
-  jukeboxtime->day = tm->tm_mday;
-  jukeboxtime->weekday = tm->tm_wday-1;
-  jukeboxtime->hours = tm->tm_hour;
-  jukeboxtime->minutes = tm->tm_min;
-  jukeboxtime->seconds = tm->tm_sec;
+  if (use_mtp) {
+    // FIXME: implement for MTP if device can set time.
+  } else {
+    njb_time_t *jukeboxtime;
+    struct tm *tm;
+  
+    // Get the time from the jukebox, then modify it.
+    jukeboxtime = NJB_Get_Time(pdedevice);
 
-  NJB_Set_Time(pdedevice, jukeboxtime);
-  NJB_Destroy_Time(jukeboxtime);
+    // Uses UNIX library calls, might need to be portablized
+    tm = localtime((time_t *) &currenttime);
+    /*
+      g_print("Setting: %d-%d-%d (wkd: %d) %d:%d:%d\n", 1900+tm->tm_year,
+      1+tm->tm_mon, tm->tm_mday, tm->tm_wday-1,
+      tm->tm_hour, tm->tm_min, tm->tm_sec);
+    */
+    jukeboxtime->year = 1900+tm->tm_year;
+    jukeboxtime->month = 1+tm->tm_mon;
+    jukeboxtime->day = tm->tm_mday;
+    jukeboxtime->weekday = tm->tm_wday-1;
+    jukeboxtime->hours = tm->tm_hour;
+    jukeboxtime->minutes = tm->tm_min;
+    jukeboxtime->seconds = tm->tm_sec;
+    
+    NJB_Set_Time(pdedevice, jukeboxtime);
+    NJB_Destroy_Time(jukeboxtime);
+  }
 }
 
 /* 
@@ -2992,6 +3384,12 @@ void jukebox_create_folder(gchar *foldername)
   jbmeta->folder = g_strdup_printf("%s%s\\", folder, foldername);
   jbmeta->filename = g_strdup(foldername);
   jbmeta->size = 0;
+
+  if (use_mtp) {
+    // FIXME: implement for MTP.
+    return;
+  }
+
   if (NJB_Create_Folder (pdedevice, jbmeta->folder, &id) == -1) {
     // Error back off
     destroy_metadata_t(jbmeta);
