@@ -438,6 +438,11 @@ void create_player_window(GList *metalist)
   GtkWidget *hbox, *hbox2, *hbox3, *vbox;
   GtkObject *adj;
 
+  // Playback on MTP devices not supported. (Yet.)
+  if (jukebox_is_mtp()) {
+    create_error_dialog(_("Playback is not supported on MTP devices"));
+    return;
+  }
   play_thread_args.metalist = metalist;
   dialog = gtk_dialog_new();
   gtk_window_set_title (GTK_WINDOW (dialog), (_("Playing tracks on the jukebox")));
