@@ -22,7 +22,12 @@ Boston, MA 02111-1307, USA.
 #include "player.h"
 #include "util.h"
 #include "prefs.h"
+
+#if !GTK_CHECK_VERSION(2,6,0)
 #include <libgnomeui/libgnomeui.h>
+#else
+#define GNOME_DIALOG GTK_DIALOG
+#endif
 
 /* Local variables */
 static GtkWidget *new_pl_dialog;
@@ -130,6 +135,15 @@ static void update_playlistid_in_store(guint old_plid, guint new_plid)
  * CALLBACK FUNCTIONS FOR GUI
  ***************************************************************************************/
 
+#if GTK_CHECK_VERSION(2,6,0)
+/* Deletes the dialog windows, passed as data */
+GCallback dispose_of_dialog_window(GtkButton * button, gpointer data)
+{
+	GtkWidget * tmpwid=GTK_WIDGET(data);
+	gtk_widget_destroy(tmpwid);
+}
+#endif
+
 static GCallback create_new_playlist_ok_button (GtkButton *button,
 						gpointer data)
 {
@@ -147,7 +161,7 @@ static GCallback create_new_playlist_ok_button (GtkButton *button,
 static GCallback playlist_new_response(gpointer data)
 {
   GtkWidget *label, *button;
-
+#if !GTK_CHECK_VERSION(2,6,0)  
   new_pl_dialog = gnome_dialog_new(_("Create a new playlist"),
 				   GNOME_STOCK_BUTTON_CANCEL,
 				   GNOME_STOCK_BUTTON_OK,
@@ -160,6 +174,30 @@ static GCallback playlist_new_response(gpointer data)
 			      1,
 			      GTK_SIGNAL_FUNC(create_new_playlist_ok_button),
 			      NULL);
+#else
+  new_pl_dialog=gtk_dialog_new_with_buttons(_("Create a new playlist"),
+		  NULL, 
+		  0, 
+		  NULL);
+
+  button = gtk_dialog_add_button(GTK_DIALOG(new_pl_dialog), 
+		  GTK_STOCK_CANCEL, 
+		  0);
+  g_signal_connect(button, 
+		  "clicked", 
+		  GTK_SIGNAL_FUNC(dispose_of_dialog_window), 
+		  GTK_OBJECT(new_pl_dialog));
+
+  button = gtk_dialog_add_button(GTK_DIALOG(new_pl_dialog), 
+		  GTK_STOCK_OK, 
+		  1);
+  g_signal_connect(button, 
+		  "clicked", 
+		  GTK_SIGNAL_FUNC(create_new_playlist_ok_button), 
+		  NULL);
+#endif
+	      
+  
   gtk_window_set_position (GTK_WINDOW(new_pl_dialog), GTK_WIN_POS_MOUSE);
 
   label = gtk_label_new(_("Choose a name for the new playlist:"));
@@ -221,7 +259,7 @@ static GCallback playlist_edit_response(gpointer data)
   guint8 dummy1;
   GdkPixmap *dummy2;
   gboolean dummy3;
-
+#if !GTK_CHECK_VERSION(2,6,0)
   edit_pl_dialog = gnome_dialog_new(_("Edit playlist"),
 				    GNOME_STOCK_BUTTON_CANCEL,
 				    GNOME_STOCK_BUTTON_OK,
@@ -234,7 +272,29 @@ static GCallback playlist_edit_response(gpointer data)
 			      1,
 			      GTK_SIGNAL_FUNC(edit_playlist_ok_button),
 			      NULL);
-  gtk_window_set_position (GTK_WINDOW(edit_pl_dialog), GTK_WIN_POS_MOUSE);
+ #else
+  edit_pl_dialog=gtk_dialog_new_with_buttons(_("Edit playlist"),
+		  NULL, 
+		  0, 
+		  NULL);
+
+  button = gtk_dialog_add_button(GTK_DIALOG(edit_pl_dialog), 
+		  GTK_STOCK_CANCEL, 
+		  0);
+  g_signal_connect(button, 
+		  "clicked", 
+		  GTK_SIGNAL_FUNC(dispose_of_dialog_window), 
+		  GTK_OBJECT(edit_pl_dialog));
+
+  button = gtk_dialog_add_button(GTK_DIALOG(edit_pl_dialog), 
+		  GTK_STOCK_OK, 
+		  1);
+  g_signal_connect(button, 
+		  "clicked", 
+		  GTK_SIGNAL_FUNC(edit_playlist_ok_button), 
+		  NULL);
+#endif
+ gtk_window_set_position (GTK_WINDOW(edit_pl_dialog), GTK_WIN_POS_MOUSE);
 
   label = gtk_label_new(_("Edit the name for this playlist:"));
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(edit_pl_dialog)->vbox), label, TRUE, TRUE, 0);
@@ -400,6 +460,7 @@ static GCallback playlist_export_response(gpointer data)
     gchar *text;
 
     if (playlist != NULL) {
+#if !GTK_CHECK_VERSION(2,6,0)
       export_pl_dialog = gnome_dialog_new(_("Export playlist"),
 					  GNOME_STOCK_BUTTON_CANCEL,
 					  GNOME_STOCK_BUTTON_OK,
@@ -412,6 +473,29 @@ static GCallback playlist_export_response(gpointer data)
 				  1,
 				  GTK_SIGNAL_FUNC(export_playlist_ok_button),
 				  playlist);
+ #else
+      GtkWidget * button;
+      export_pl_dialog=gtk_dialog_new_with_buttons(_("Export playlist"),
+		  NULL, 
+		  0, 
+		  NULL);
+
+      button = gtk_dialog_add_button(GTK_DIALOG(export_pl_dialog), 
+		  GTK_STOCK_CANCEL, 
+		  0);
+      g_signal_connect(button, 
+		  "clicked", 
+		  GTK_SIGNAL_FUNC(dispose_of_dialog_window), 
+		  GTK_OBJECT(export_pl_dialog));
+
+      button = gtk_dialog_add_button(GTK_DIALOG(export_pl_dialog), 
+		  GTK_STOCK_OK, 
+		  1);
+      g_signal_connect(button, 
+		  "clicked", 
+		  GTK_SIGNAL_FUNC(export_playlist_ok_button), 
+		  playlist);
+#endif
       gtk_window_set_position (GTK_WINDOW(export_pl_dialog), GTK_WIN_POS_MOUSE);
       
       label = gtk_label_new(_("Export playlist to this file:"));
@@ -645,7 +729,7 @@ static void playlist_selection_dialog(gchar *title, gchar *prompt, GtkSignalFunc
     g_list_free(selected_target_playlists);
     selected_target_playlists = NULL;
   }
-
+#if !GTK_CHECK_VERSION(2,6,0)
   select_playlist_dialog = gnome_dialog_new(title,
 					    GNOME_STOCK_BUTTON_CANCEL,
 					    GNOME_STOCK_BUTTON_OK,
@@ -658,6 +742,28 @@ static void playlist_selection_dialog(gchar *title, gchar *prompt, GtkSignalFunc
 			      1,
 			      G_CALLBACK(ok_signal),
 			      NULL);
+#else
+  select_playlist_dialog=gtk_dialog_new_with_buttons(title,
+		  NULL, 
+		  0, 
+		  NULL);
+
+  button = gtk_dialog_add_button(GTK_DIALOG(select_playlist_dialog), 
+		  GTK_STOCK_CANCEL, 
+		  0);
+  g_signal_connect(button, 
+		  "clicked", 
+		  GTK_SIGNAL_FUNC(dispose_of_dialog_window), 
+		  GTK_OBJECT(select_playlist_dialog));
+
+  button = gtk_dialog_add_button(GTK_DIALOG(select_playlist_dialog), 
+		  GTK_STOCK_OK, 
+		  1);
+  g_signal_connect(button, 
+		  "clicked", 
+		  GTK_SIGNAL_FUNC(ok_signal), 
+		  NULL);
+#endif
   gtk_window_set_position (GTK_WINDOW(select_playlist_dialog), GTK_WIN_POS_MOUSE);
   g_signal_connect_object(GTK_OBJECT (select_playlist_dialog),
 			  "delete_event",
