@@ -26,7 +26,9 @@ static GtkWidget *id3remove = NULL;
 static GtkWidget *useid3 = NULL;
 static GtkWidget *useid3_unicode = NULL;
 static GtkWidget *display_dotdirs = NULL;
+#ifndef G_OS_WIN32
 static GtkWidget *force_8859 = NULL;
+#endif
 static GtkWidget *filenameformat = NULL;
 static GtkWidget *use_origname = NULL;
 static GtkWidget *ask_playlist = NULL;
@@ -36,6 +38,7 @@ static GtkWidget *extended_metadata = NULL;
 static GtkWidget *autoscan = NULL;
 static GtkWidget *set_time = NULL;
 static GtkWidget *remember_dir = NULL;
+static GtkWidget *turbo_mode = NULL;
 
 static void read_prefs_file()
 {
@@ -56,9 +59,7 @@ static void read_prefs_file()
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autoscan), TRUE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(set_time), TRUE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(remember_dir), TRUE);
-  /* People must learn to use Unicode! Stop forcing this. */
-  /* gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_8859), TRUE); */
-
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(turbo_mode), TRUE);
 
   /* g_print("Called read prefs\n"); */
   scanner = g_scanner_new(NULL);
@@ -125,11 +126,13 @@ static void read_prefs_file()
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(recurse_dir), TRUE);
 	else
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(recurse_dir), FALSE);
+#ifndef G_OS_WIN32
       } else if (!strcmp(name, "FORCE_8859")) {
 	if (g_scanner_cur_value(scanner).v_int)
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_8859), TRUE);
 	else
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_8859), FALSE);
+#endif
       } else if (!strcmp(name, "DETECT_METADATA")) {
 	if (g_scanner_cur_value(scanner).v_int)
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(detect_metadata), TRUE);
@@ -155,6 +158,11 @@ static void read_prefs_file()
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(remember_dir), TRUE);
 	else
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(remember_dir), FALSE);
+      } else if (!strcmp(name, "TURBO_MODE")) {
+	if (g_scanner_cur_value(scanner).v_int)
+	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(turbo_mode), TRUE);
+	else
+	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(turbo_mode), FALSE);
       } else if (!strcmp(name, "CURRENT_DIRECTORY")) {
 	if (get_prefs_remember_dir() && is_directory(g_scanner_cur_value(scanner).v_string)) {
 	  change_directory(g_scanner_cur_value(scanner).v_string);
@@ -230,12 +238,14 @@ void write_prefs_file()
   else
     prefs = stringcat(prefs, "0");
   prefs = stringcat(prefs, "\n");
+#ifndef G_OS_WIN32
   prefs = stringcat(prefs, "FORCE_8859=");
   if (get_prefs_force_8859())
     prefs = stringcat(prefs, "1");
   else
     prefs = stringcat(prefs, "0");
   prefs = stringcat(prefs, "\n");
+#endif
   prefs = stringcat(prefs, "DETECT_METADATA=");
   if (get_prefs_detect_metadata())
     prefs = stringcat(prefs, "1");
@@ -266,6 +276,12 @@ void write_prefs_file()
   else
     prefs = stringcat(prefs, "0");
   prefs = stringcat(prefs, "\n");
+  prefs = stringcat(prefs, "TURBO_MODE=");
+  if (get_prefs_turbo_mode())
+    prefs = stringcat(prefs, "1");
+  else
+    prefs = stringcat(prefs, "0");
+  prefs = stringcat(prefs, "\n");
 
   prefs = stringcat(prefs, "CURRENT_DIRECTORY='");
   tmp = get_current_dir();
@@ -340,7 +356,7 @@ gboolean get_prefs_recurse_dir()
     return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(recurse_dir));
 }
 
-
+#ifndef G_OS_WIN32
 gboolean get_prefs_force_8859()
 {
   if (force_8859 == NULL)
@@ -348,6 +364,7 @@ gboolean get_prefs_force_8859()
   else
     return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(force_8859));
 }
+#endif
 
 gboolean get_prefs_detect_metadata()
 {
@@ -389,6 +406,15 @@ gboolean get_prefs_remember_dir()
     return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(remember_dir));
 }
 
+gboolean get_prefs_turbo_mode()
+{
+  if (turbo_mode == NULL)
+    return FALSE;
+  else
+    return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(turbo_mode));
+}
+
+
 gchar *get_prefs_filenameformat()
 {
   static gchar defaultformat[] = "%a - %t";
@@ -485,8 +511,10 @@ void create_prefs_widgets(GtkWidget *box)
     gtk_check_button_new_with_label(_("Ask to add files to a playlist"));
   recurse_dir =
     gtk_check_button_new_with_label(_("Recurse into any selected directories when transferring"));
+#ifndef G_OS_WIN32
   force_8859 =
     gtk_check_button_new_with_label(_("Force local filesystem to ISO-8859-1 charset"));
+#endif
   detect_metadata =
     gtk_check_button_new_with_label(_("Try to extract sound file metadata from path and filename"));
   extended_metadata =
@@ -502,6 +530,8 @@ void create_prefs_widgets(GtkWidget *box)
     gtk_check_button_new_with_label(_("Automatically set the time on the Jukebox to that of your computer on startup"));
   remember_dir =
     gtk_check_button_new_with_label(_("On startup, return to the place in the filesystem where you were last time"));
+  turbo_mode =
+    gtk_check_button_new_with_label(_("Enable turbo mode - uncheck this if you have problems"));
   gtk_box_pack_start(GTK_BOX(vbox), id3override, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), id3remove, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), useid3, FALSE, FALSE, 0);
@@ -509,12 +539,15 @@ void create_prefs_widgets(GtkWidget *box)
   gtk_box_pack_start(GTK_BOX(vbox), display_dotdirs, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), ask_playlist, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), recurse_dir, FALSE, FALSE, 0);
+#ifndef G_OS_WIN32
   gtk_box_pack_start(GTK_BOX(vbox), force_8859, FALSE, FALSE, 0);
+#endif
   gtk_box_pack_start(GTK_BOX(vbox), detect_metadata, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), extended_metadata, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), autoscan, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), set_time, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), remember_dir, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), turbo_mode, FALSE, FALSE, 0);
   gtk_widget_show(id3override);
   gtk_widget_show(id3remove);
   gtk_widget_show(useid3);
@@ -522,12 +555,15 @@ void create_prefs_widgets(GtkWidget *box)
   gtk_widget_show(display_dotdirs);
   gtk_widget_show(ask_playlist);
   gtk_widget_show(recurse_dir);
+#ifndef G_OS_WIN32
   gtk_widget_show(force_8859);
+#endif
   gtk_widget_show(detect_metadata);
   gtk_widget_show(extended_metadata);
   gtk_widget_show(autoscan);
   gtk_widget_show(set_time);
   gtk_widget_show(remember_dir);
+  gtk_widget_show(turbo_mode);
 
   gtk_widget_show(vbox);
   gtk_box_pack_start(GTK_BOX(box), vbox, TRUE, TRUE, 0);
