@@ -275,7 +275,11 @@ get_tag_for_wmafile (metadata_t *meta)
   gint n;
 
   // g_print("Getting WMA tag info for %s...\n", meta->path);
-  fd = (gint) open(tmppath, O_RDONLY, 0);
+#if !GLIB_CHECK_VERSION(2,6,0)
+  fd = (gint) open(tmppath, READONLY_FLAGS, 0);
+#else
+  fd = (gint) g_open(tmppath, READONLY_FLAGS, 0);
+#endif
   if (fd < 0) {
     g_free(tmppath);
     return;
