@@ -42,9 +42,11 @@ Boston, MA 02111-1307, USA.
 static void transfer_from_jukebox_to_hd (GtkButton *button,
 		      gpointer data)
 {
-  static jb2hd_thread_arg_t jb2hd_thread_args;
+  jb2hd_thread_arg_t *jb2hd_thread_args;
   GList *metalist = get_all_metadata_from_selection(JB_LIST);
 
+  jb2hd_thread_args = (jb2hd_thread_arg_t *) malloc(sizeof (jb2hd_thread_arg_t));
+
   if (jukebox_locked) {
     create_error_dialog(_("Jukebox busy"));
     destroy_metalist(metalist);
@@ -92,12 +94,12 @@ static void transfer_from_jukebox_to_hd (GtkButton *button,
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), progress_bar, TRUE, TRUE, 0);
     gtk_widget_show_all(dialog);
 
-    jb2hd_thread_args.dialog = dialog;
-    jb2hd_thread_args.label = label2;
-    jb2hd_thread_args.metalist = metalist;
+    jb2hd_thread_args->dialog = dialog;
+    jb2hd_thread_args->label = label2;
+    jb2hd_thread_args->metalist = metalist;
     jukebox_locked = TRUE;
     cancel_jukebox_operation = FALSE;
-    g_thread_create(jb2hd_thread,(gpointer) &jb2hd_thread_args, FALSE, NULL);
+    g_thread_create(jb2hd_thread,(gpointer) jb2hd_thread_args, FALSE, NULL);
   }
 }
 
@@ -154,7 +156,6 @@ void transfer_from_hd_to_jukebox(GList *playlists)
     hd2jb_thread_arg_t *hd2jb_thread_args;
     GtkWidget *label1, *label2, *dialog, *button, *separator;
 
-
     hd2jb_thread_args = (hd2jb_thread_arg_t *) malloc(sizeof (hd2jb_thread_arg_t));
 #if !GTK_CHECK_VERSION(2,6,0)
     dialog = gnome_dialog_new(_("Transfering tracks to jukebox library"),
